############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
############################################################
import logging
import PyQt5
import time
import copy
from queue import Queue
from astrometry import transform


class MountGetModelNames(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    CONNECTION_TIMEOUT = 2000
    CYCLE = 250
    signalDestruct = PyQt5.QtCore.pyqtSignal()

    def __init__(self, app, thread, data, signalConnected, mountStatus):
        super().__init__()

        self.app = app
        self.thread = thread
        self.data = data
        self.signalConnected = signalConnected
        self.mountStatus = mountStatus
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        self.isRunning = False
        self.connectCounter = 0
        self.socket = None
        self.sendLock = False
        self.cycleTimer = None
        self.messageString = ''
        self.sendCommandQueue = Queue()

    def run(self):
        self.logger.info('mount get model names started')
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.socket = PyQt5.QtNetwork.QTcpSocket()
        self.socket.setSocketOption(PyQt5.QtNetwork.QAbstractSocket.LowDelayOption, 1)
        self.socket.setSocketOption(PyQt5.QtNetwork.QAbstractSocket.KeepAliveOption, 1)
        self.socket.hostFound.connect(self.handleHostFound)
        self.socket.connected.connect(self.handleConnected)
        self.socket.stateChanged.connect(self.handleStateChanged)
        self.socket.disconnected.connect(self.handleDisconnect)
        self.socket.readyRead.connect(self.handleReadyRead)
        self.socket.error.connect(self.handleError)

        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.signalConnected.emit({'GetName': False})
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        self.logger.info('mount get model names stopped')

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
            self.socket.disconnectFromHost()
        self.cycleTimer.stop()
        self.signalDestruct.disconnect(self.destruct)
        self.socket.hostFound.disconnect(self.handleHostFound)
        self.socket.connected.disconnect(self.handleConnected)
        self.socket.stateChanged.disconnect(self.handleStateChanged)
        self.socket.disconnected.disconnect(self.handleDisconnect)
        self.socket.error.disconnect(self.handleError)
        self.socket.readyRead.disconnect(self.handleReadyRead)
        self.socket.abort()

    def doCommand(self):
        self.doReconnect()
        if not self.sendCommandQueue.empty() and (self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState):
            command = self.sendCommandQueue.get()
            if not self.sendLock:
                self.sendCommand(command)

    def doReconnect(self):
        # to get order in connections, we wait for first connecting the once type
        if self.mountStatus['Once'] and self.data['FW'] > 0:
            if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.UnconnectedState:
                if self.connectCounter == 0:
                    self.app.sharedMountDataLock.lockForRead()
                    self.socket.connectToHost(self.data['MountIP'], self.data['MountPort'])
                    self.app.sharedMountDataLock.unlock()
                    self.sendCommandQueue.queue.clear()
                else:
                    # connection build up is ongoing
                    pass
                if self.connectCounter * self.CYCLE > self.CONNECTION_TIMEOUT:
                    self.socket.abort()
                    self.connectCounter = 0
                else:
                    self.connectCounter += 1
            else:
                if self.socket.state() != PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
                    if self.connectCounter * self.CYCLE > self.CONNECTION_TIMEOUT:
                        self.socket.abort()
                        self.connectCounter = 0
                    else:
                        self.connectCounter += 1
                else:
                    # connected
                    pass

    @PyQt5.QtCore.pyqtSlot()
    def handleHostFound(self):
        self.logger.debug('Mount GetModelNames found at {}:{}'.format(self.data['MountIP'], self.data['MountPort']))

    @PyQt5.QtCore.pyqtSlot()
    def handleConnected(self):
        self.signalConnected.emit({'GetName': True})
        self.getModelNames()
        self.logger.info('Mount GetModelNames connected at {0}:{1}'.format(self.data['MountIP'], self.data['MountPort']))

    @PyQt5.QtCore.pyqtSlot(PyQt5.QtNetwork.QAbstractSocket.SocketError)
    def handleError(self, socketError):
        self.logger.warning('Mount GetModelNames connection fault: {0}'.format(socketError))

    @PyQt5.QtCore.pyqtSlot()
    def handleStateChanged(self):
        self.logger.debug('Mount GetModelNames connection has state: {0}'.format(self.socket.state()))

    @PyQt5.QtCore.pyqtSlot()
    def handleDisconnect(self):
        self.logger.info('Mount GetModelNames connection is disconnected from host')
        self.signalConnected.emit({'GetName': False})

    def sendCommand(self, command):
        if self.isRunning:
            if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
                self.sendLock = True
                self.socket.write(bytes(command + '\r', encoding='ascii'))
                self.socket.flush()
            else:
                self.sendLock = False
                self.logger.warning('Socket GetModelNames not connected')

    def getModelNames(self):
        # asking for 50 model names
        command = ''
        for i in range(1, 51):
            command += (':modelnam{0:d}#'.format(i))
        self.sendCommandQueue.put(command)

    @PyQt5.QtCore.pyqtSlot()
    def handleReadyRead(self):
        # Get message from socket.
        while self.socket.bytesAvailable() and self.isRunning:
            self.messageString += self.socket.read(4000).decode()

        if self.messageString.count('#') != 50:
            if self.messageString.count('#') > 50:
                self.logger.error('Receiving data got error:{0}'.format(self.messageString))
                messageToProcess = self.messageString
                self.messageString = ''
            else:
                # go on receiving data
                return
        else:
            messageToProcess = self.messageString
            self.messageString = ''
        # now we got all information about the model write run
        valueList = messageToProcess.strip('#').split('#')
        # quick check:
        self.app.sharedMountDataLock.lockForWrite()
        self.data['ModelNames'] = copy.copy(valueList)
        self.app.sharedMountDataLock.unlock()
        self.app.workerMountDispatcher.signalMountShowModelNames.emit()
        self.sendLock = False



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
############################################################
import logging
import PyQt5
import time
from queue import Queue
from astrometry import transform


class MountGetAlignmentModel(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    CONNECTION_TIMEOUT = 2000
    CYCLE = 250
    signalDestruct = PyQt5.QtCore.pyqtSignal()

    def __init__(self, app, thread, data, signalConnected, mountStatus):
        super().__init__()

        self.app = app
        self.thread = thread
        self.data = data
        self.signalConnected = signalConnected
        self.mountStatus = mountStatus
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        self.isRunning = False
        self.connectCounter = 0
        self.socket = None
        self.sendLock = False
        self.cycleTimer = None
        self.messageString = ''
        self.sendCommandQueue = Queue()
        self.transform = transform.Transform(self.app)

    def run(self):
        self.logger.info('mount get align started')
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.socket = PyQt5.QtNetwork.QTcpSocket()
        self.socket.setSocketOption(PyQt5.QtNetwork.QAbstractSocket.LowDelayOption, 1)
        self.socket.setSocketOption(PyQt5.QtNetwork.QAbstractSocket.KeepAliveOption, 1)
        self.socket.hostFound.connect(self.handleHostFound)
        self.socket.connected.connect(self.handleConnected)
        self.socket.stateChanged.connect(self.handleStateChanged)
        self.socket.disconnected.connect(self.handleDisconnect)
        self.socket.readyRead.connect(self.handleReadyRead)
        self.socket.error.connect(self.handleError)

        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.signalConnected.emit({'GetAlign': False})
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        self.logger.info('mount get align stopped')

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
            self.socket.disconnectFromHost()
        self.cycleTimer.stop()
        self.signalDestruct.disconnect(self.destruct)
        self.socket.hostFound.disconnect(self.handleHostFound)
        self.socket.connected.disconnect(self.handleConnected)
        self.socket.stateChanged.disconnect(self.handleStateChanged)
        self.socket.disconnected.disconnect(self.handleDisconnect)
        self.socket.error.disconnect(self.handleError)
        self.socket.readyRead.disconnect(self.handleReadyRead)
        self.socket.abort()

    def doCommand(self):
        self.doReconnect()
        if not self.sendCommandQueue.empty() and (self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState):
            command = self.sendCommandQueue.get()
            if not self.sendLock:
                self.sendCommand(command)

    def doReconnect(self):
        # to get order in connections, we wait for first connecting the once type
        if self.mountStatus['Once'] and self.data['FW'] > 0:
            if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.UnconnectedState:
                if self.connectCounter == 0:
                    self.app.sharedMountDataLock.lockForRead()
                    self.socket.connectToHost(self.data['MountIP'], self.data['MountPort'])
                    self.app.sharedMountDataLock.unlock()
                    self.sendCommandQueue.queue.clear()
                else:
                    # connection build up is ongoing
                    pass
                if self.connectCounter * self.CYCLE > self.CONNECTION_TIMEOUT:
                    self.socket.abort()
                    self.connectCounter = 0
                else:
                    self.connectCounter += 1
            else:
                if self.socket.state() != PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
                    if self.connectCounter * self.CYCLE > self.CONNECTION_TIMEOUT:
                        self.socket.abort()
                        self.connectCounter = 0
                    else:
                        self.connectCounter += 1
                else:
                    # connected
                    pass

    @PyQt5.QtCore.pyqtSlot()
    def handleHostFound(self):
        self.logger.debug('Mount GetAlignmentModel found at {}:{}'.format(self.data['MountIP'], self.data['MountPort']))

    @PyQt5.QtCore.pyqtSlot()
    def handleConnected(self):
        self.signalConnected.emit({'GetAlign': True})
        self.logger.info('Mount GetAlignmentModel connected at {0}:{1}'.format(self.data['MountIP'], self.data['MountPort']))
        self.getAlignmentModel()

    @PyQt5.QtCore.pyqtSlot(PyQt5.QtNetwork.QAbstractSocket.SocketError)
    def handleError(self, socketError):
        self.logger.warning('Mount GetAlignmentModel connection fault: {0}'.format(socketError))

    @PyQt5.QtCore.pyqtSlot()
    def handleStateChanged(self):
        self.logger.debug('Mount GetAlignmentModel connection has state: {0}'.format(self.socket.state()))

    @PyQt5.QtCore.pyqtSlot()
    def handleDisconnect(self):
        self.logger.info('Mount GetAlignmentModel connection is disconnected from host')
        self.signalConnected.emit({'GetAlign': False})

    def sendCommand(self, command):
        if self.isRunning:
            if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
                self.sendLock = True
                self.socket.write(bytes(command + '\r', encoding='ascii'))
                self.socket.flush()
            else:
                self.sendLock = False
                self.logger.warning('Socket GetAlignmentModel not connected')

    def getAlignmentModel(self):
        self.data['ModelLoading'] = True
        if self.data['FW'] < 21500:
            command = ':getalst#'
        else:
            command = ':getalst#:getain#'
        # asking for 100 points data
        for i in range(1, 102):
            command += (':getalp{0:d}#'.format(i))
        self.sendCommandQueue.put(command)

    @PyQt5.QtCore.pyqtSlot()
    def handleReadyRead(self):
        # Get message from socket.
        while self.socket.bytesAvailable() and self.isRunning:
            self.messageString += self.socket.read(4000).decode()
        # if the last characters are not E#, there are more points to receive
        if not self.messageString.endswith('E#'):
            return
        else:
            messageToProcess = self.messageString
            self.messageString = ''
        while messageToProcess.endswith('E#'):
            messageToProcess = messageToProcess.rstrip('E#')
        while messageToProcess.startswith('E#'):
            messageToProcess = messageToProcess.lstrip('E#')
        # now transfer the model data
        try:
            if len(messageToProcess) == 0:
                return
            self.app.sharedMountDataLock.lockForWrite()
            self.logger.info('Raw data from Mount: {0}'.format(messageToProcess))
            valueList = messageToProcess.strip('#').split('#')
            # now the first part of the command cluster
            numberStars = int(valueList[0])
            self.logger.info('Align info number stars: {0}'.format(numberStars))
            self.data['NumberAlignmentStars'] = numberStars
            self.data['Number'] = numberStars
            del valueList[0]
            if numberStars < 3:
                valueList = ['E,E,E,E,E,E,E,E,E']
            self.logger.info('Align info data: {0}'.format(valueList[0]))
            # now the second part of the command cluster. it is related to firmware feature
            if self.data['FW'] > 21500:
                if numberStars < 3:
                    valueList = ['E,E,E,E,E,E,E,E,E']
                # here we have more data in
                if len(valueList[0]) > 3:
                    a1, a2, a3, a4, a5, a6, a7, a8, a9 = valueList[0].split(',')
                    # 'E' could be sent if not calculable or no value available
                    if a1 != 'E':
                        self.data['ModelErrorAzimuth'] = float(a1)
                    else:
                        self.data['ModelErrorAzimuth'] = 0
                    if a2 != 'E':
                        self.data['ModelErrorAltitude'] = float(a2)
                    else:
                        self.data['ModelErrorAltitude'] = 0
                    if a3 != 'E':
                        self.data['PolarError'] = float(a3)
                    else:
                        self.data['PolarError'] = 0
                    if a4 != 'E':
                        self.data['PosAngle'] = float(a4)
                    else:
                        self.data['PosAngle'] = 0
                    if a5 != 'E':
                        self.data['OrthoError'] = float(a5)
                    else:
                        self.data['OrthoError'] = 0
                    if a6 != 'E':
                        self.data['AzimuthKnobs'] = float(a6)
                    else:
                        self.data['AzimuthKnobs'] = 0
                    if a7 != 'E':
                        self.data['AltitudeKnobs'] = float(a7)
                    else:
                        self.data['AltitudeKnobs'] = 0
                    if a8 != 'E':
                        self.data['Terms'] = int(a8)
                    else:
                        self.data['Terms'] = 0
                    if a9 != 'E':
                        self.data['RMS'] = float(a9)
                    else:
                        self.data['RMS'] = 0
                    self.data['ModelRMSError'] = '{0:3.1f}'.format(self.data['RMS'])
                    self.data['ModelErrorPosAngle'] = '{0:3.1f}'.format(self.data['PosAngle'])
                    self.data['ModelPolarError'] = '{0}'.format(self.transform.decimalToDegree(self.data['PolarError']))
                    self.data['ModelOrthoError'] = '{0}'.format(self.transform.decimalToDegree(self.data['OrthoError']))
                    self.data['ModelErrorAz'] = '{0}'.format(self.transform.decimalToDegree(self.data['ModelErrorAzimuth']))
                    self.data['ModelErrorAlt'] = '{0}'.format(self.transform.decimalToDegree(self.data['ModelErrorAltitude']))
                    self.data['ModelTerms'] = '{0:2d}'.format(self.data['Terms'])
                    if self.data['AzimuthKnobs'] > 0:
                        value = '{0:2.2f} left'.format(abs(self.data['AzimuthKnobs']))
                    else:
                        value = '{0:2.2f} right'.format(abs(self.data['AzimuthKnobs']))
                    self.data['ModelKnobTurnAz'] = '{0}'.format(value)
                    if self.data['AltitudeKnobs'] > 0:
                        value = '{0:2.2f} down'.format(abs(self.data['AltitudeKnobs']))
                    else:
                        value = '{0:2.2f} up'.format(abs(self.data['AltitudeKnobs']))
                    self.data['ModelKnobTurnAlt'] = '{0}'.format(value)
                else:
                    self.logger.error('Receive error getain command content: {0}'.format(valueList[0]))
                # remove the first remaining element in list if it was there
                del valueList[0]
            # now the third part of the command cluster
            self.data['ModelIndex'] = list()
            self.data['ModelAzimuth'] = list()
            self.data['ModelAltitude'] = list()
            self.data['ModelError'] = list()
            self.data['ModelErrorAngle'] = list()
            # we start every time with index 0, because if the first parsing took place, the first list element will be deleted
            self.logger.info('Align info points data: {0}'.format(valueList))
            for i in range(0, len(valueList)):
                values = valueList[i].split(',')
                ha = values[0]
                dec = values[1]
                ErrorRMS = float(values[2])
                ErrorAngle = float(values[3])
                dec = dec.replace('*', ':')
                RaJNow = self.transform.degStringToDecimal(ha)
                DecJNow = self.transform.degStringToDecimal(dec)
                az, alt = self.transform.topocentricToAzAlt(RaJNow, DecJNow)
                # index should start with 0, but numbering in mount starts with 1
                self.data['ModelIndex'].append(i)
                self.data['ModelAzimuth'].append(az)
                self.data['ModelAltitude'].append(alt)
                self.data['ModelError'].append(ErrorRMS)
                self.data['ModelErrorAngle'].append(ErrorAngle)
        except Exception as e:
            self.logger.error('Parsing GetAlignmentModel got error:{0}, values:{1}'.format(e, messageToProcess))
        finally:
            self.app.sharedMountDataLock.unlock()
            self.app.workerMountDispatcher.signalMountShowAlignmentModel.emit()
        self.data['ModelLoading'] = False
        self.sendLock = False



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import PyQt5
import time
from baseclasses import checkIP


class Audio(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    signalDestruct = PyQt5.QtCore.pyqtSignal()
    CYCLE = 500

    def __init__(self, app, thread):
        super().__init__()
        self.isRunning = False
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        self.mutexIPChanged = PyQt5.QtCore.QMutex()

        self.app = app
        self.thread = thread
        self.cycleTimer = None

        # define audio signals
        self.audioSignalsSet = dict()
        self.guiAudioList = dict()
        self.prepareGui()

    def initConfig(self):
        try:
            if 'PlayMountSlew' in self.app.config:
                self.app.ui.soundMountSlewFinished.setCurrentIndex(self.app.config['PlayMountSlew'])
            if 'PlayDomeSlew' in self.app.config:
                self.app.ui.soundDomeSlewFinished.setCurrentIndex(self.app.config['PlayDomeSlew'])
            if 'PlayMountAlert' in self.app.config:
                self.app.ui.soundMountAlert.setCurrentIndex(self.app.config['PlayMountAlert'])
            if 'PlayModelingFinished' in self.app.config:
                self.app.ui.soundModelingFinished.setCurrentIndex(self.app.config['PlayModelingFinished'])
        except Exception as e:
            self.logger.error('item in config.cfg for audio could not be initialized, error:{0}'.format(e))
        finally:
            pass

    def storeConfig(self):
        self.app.config['PlayMountSlew'] = self.app.ui.soundMountSlewFinished.currentIndex()
        self.app.config['PlayDomeSlew'] = self.app.ui.soundDomeSlewFinished.currentIndex()
        self.app.config['PlayMountAlert'] = self.app.ui.soundMountAlert.currentIndex()
        self.app.config['PlayModelingFinished'] = self.app.ui.soundModelingFinished.currentIndex()

    def run(self):
        # a running thread is shown with variable isRunning = True. This thread should hav it's own event loop
        self.logger.info('audio started')
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.setupAudioSignals()
        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        self.logger.info('audio stopped')

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        self.signalDestruct.disconnect(self.destruct)
        self.cycleTimer.stop()

    def doCommand(self):
        if not self.app.audioCommandQueue.empty():
            command = self.app.audioCommandQueue.get()
            self.playAudioSignal(command)

    def prepareGui(self):
        self.guiAudioList = dict()
        # adding the possible sounds to drop down menu
        self.guiAudioList['MountSlew'] = self.app.ui.soundMountSlewFinished
        self.guiAudioList['DomeSlew'] = self.app.ui.soundDomeSlewFinished
        self.guiAudioList['MountAlert'] = self.app.ui.soundMountAlert
        self.guiAudioList['ModelingFinished'] = self.app.ui.soundModelingFinished
        for itemKey, itemValue in self.guiAudioList.items():
            self.guiAudioList[itemKey].addItem('None')
            self.guiAudioList[itemKey].addItem('Beep')
            self.guiAudioList[itemKey].addItem('Horn')
            self.guiAudioList[itemKey].addItem('Beep1')
            self.guiAudioList[itemKey].addItem('Alarm')
            self.guiAudioList[itemKey].addItem('Alert')

    def setupAudioSignals(self):
        # define audio signals
        self.audioSignalsSet = dict()
        # load the sounds available
        self.audioSignalsSet['Beep'] = PyQt5.QtMultimedia.QSound(':/beep.wav')
        self.audioSignalsSet['Alert'] = PyQt5.QtMultimedia.QSound(':/alert.wav')
        self.audioSignalsSet['Horn'] = PyQt5.QtMultimedia.QSound(':/horn.wav')
        self.audioSignalsSet['Beep1'] = PyQt5.QtMultimedia.QSound(':/beep1.wav')
        self.audioSignalsSet['Alarm'] = PyQt5.QtMultimedia.QSound(':/alarm.wav')

    def playAudioSignal(self, value):
        if value in self.guiAudioList:
            sound = self.guiAudioList[value].currentText()
            if sound in self.audioSignalsSet:
                self.audioSignalsSet[sound].play()



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
############################################################
import logging
import PyQt5
import time
from queue import Queue
from astrometry import transform
from mount import align_stars


class MountStatusRunnerSlow(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    CYCLE_STATUS_SLOW = 10000
    CYCLE = 250
    signalDestruct = PyQt5.QtCore.pyqtSignal()

    def __init__(self, app, thread, data, signalConnected, mountStatus):
        super().__init__()

        self.app = app
        self.thread = thread
        self.data = data
        self.signalConnected = signalConnected
        self.mountStatus = mountStatus
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        self.dataTimer = None
        self.cycleTimer = None
        self.isRunning = False
        self.socket = None
        self.sendLock = False
        self.messageString = ''
        self.sendCommandQueue = Queue()
        self.transform = transform.Transform(self.app)
        self.alignmentStars = align_stars.AlignStars(self.app)

        self.app.sharedMountDataLock.lockForWrite()
        self.data['starsTopo'] = list()
        self.data['starsNames'] = list()
        self.data['starsICRS'] = list()
        self.app.sharedMountDataLock.unlock()
        for name in self.alignmentStars.stars:
            self.app.sharedMountDataLock.lockForWrite()
            self.data['starsNames'].append(name)
            self.data['starsICRS'].append(self.alignmentStars.stars[name])
            self.app.sharedMountDataLock.unlock()
        self.updateAlignmentStarPositions()

    def run(self):
        self.logger.info('mount slow started')
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.socket = PyQt5.QtNetwork.QTcpSocket()
        self.socket.setSocketOption(PyQt5.QtNetwork.QAbstractSocket.LowDelayOption, 1)
        self.socket.setSocketOption(PyQt5.QtNetwork.QAbstractSocket.KeepAliveOption, 1)
        self.socket.hostFound.connect(self.handleHostFound)
        self.socket.connected.connect(self.handleConnected)
        self.socket.stateChanged.connect(self.handleStateChanged)
        self.socket.disconnected.connect(self.handleDisconnect)
        self.socket.readyRead.connect(self.handleReadyRead)
        self.socket.error.connect(self.handleError)
        # timers
        self.dataTimer = PyQt5.QtCore.QTimer(self)
        self.dataTimer.setSingleShot(False)
        self.dataTimer.timeout.connect(self.getStatusSlow)
        time.sleep(1)
        self.dataTimer.start(self.CYCLE_STATUS_SLOW)
        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.signalConnected.emit({'Slow': False})
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        self.logger.info('mount slow stopped')

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
            self.socket.disconnectFromHost()
        self.cycleTimer.stop()
        self.dataTimer.stop()
        self.signalDestruct.disconnect(self.destruct)
        self.socket.hostFound.disconnect(self.handleHostFound)
        self.socket.connected.disconnect(self.handleConnected)
        self.socket.stateChanged.disconnect(self.handleStateChanged)
        self.socket.disconnected.disconnect(self.handleDisconnect)
        self.socket.error.disconnect(self.handleError)
        self.socket.readyRead.disconnect(self.handleReadyRead)
        self.socket.abort()

    def doCommand(self):
        self.doReconnect()
        if not self.sendCommandQueue.empty() and (self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState):
            command = self.sendCommandQueue.get()
            if not self.sendLock:
                self.sendCommand(command)

    def doReconnect(self):
        # to get order in connections, we wait for first connecting the once type
        if self.mountStatus['Once'] and self.data['FW'] > 0:
            if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.UnconnectedState:
                self.app.sharedMountDataLock.lockForRead()
                self.socket.connectToHost(self.data['MountIP'], self.data['MountPort'])
                self.app.sharedMountDataLock.unlock()
                self.sendCommandQueue.queue.clear()

    @PyQt5.QtCore.pyqtSlot()
    def handleHostFound(self):
        self.app.sharedMountDataLock.lockForRead()
        self.logger.debug('Mount RunnerSlow found at {}:{}'.format(self.data['MountIP'], self.data['MountPort']))
        self.app.sharedMountDataLock.unlock()

    @PyQt5.QtCore.pyqtSlot()
    def handleConnected(self):
        self.signalConnected.emit({'Slow': True})
        self.getStatusSlow()
        self.app.sharedMountDataLock.lockForRead()
        self.logger.info('Mount RunnerSlow connected at {0}:{1}'.format(self.data['MountIP'], self.data['MountPort']))
        self.app.sharedMountDataLock.unlock()

    @PyQt5.QtCore.pyqtSlot(PyQt5.QtNetwork.QAbstractSocket.SocketError)
    def handleError(self, socketError):
        self.logger.warning('Mount RunnerSlow connection fault: {0}'.format(socketError))

    @PyQt5.QtCore.pyqtSlot()
    def handleStateChanged(self):
        self.logger.debug('Mount RunnerSlow connection has state: {0}'.format(self.socket.state()))

    @PyQt5.QtCore.pyqtSlot()
    def handleDisconnect(self):
        self.logger.info('Mount RunnerSlow connection is disconnected from host')
        self.signalConnected.emit({'Slow': False})

    def sendCommand(self, command):
        if self.isRunning:
            if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
                self.socket.write(bytes(command + '\r', encoding='ascii'))
                self.socket.flush()
            else:
                self.logger.warning('Socket RunnerSlow not connected')

    @PyQt5.QtCore.pyqtSlot()
    def getStatusSlow(self):
        if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
            self.app.sharedMountDataLock.lockForRead()
            if self.data['FW'] < 21500:
                self.sendCommandQueue.put(':U2#:GTMP1#:GREF#:Guaf#:Gdat#:Gh#:Go#')
            else:
                self.sendCommandQueue.put(':U2#:GTMP1#:GREF#:Guaf#:Gdat#:Gh#:Go#:GDUTV#')
            self.app.sharedMountDataLock.unlock()
        self.updateAlignmentStarPositions()
        self.app.workerMountDispatcher.signalAlignmentStars.emit()

    def updateAlignmentStarPositions(self):
        # update topo data for alignment stars
        self.app.sharedMountDataLock.lockForWrite()
        self.data['starsTopo'] = list()
        self.app.sharedMountDataLock.unlock()
        for name in self.alignmentStars.stars:
            self.app.sharedMountDataLock.lockForWrite()
            ra = self.transform.degStringToDecimal(self.alignmentStars.stars[name][0], ' ')
            dec = self.transform.degStringToDecimal(self.alignmentStars.stars[name][1], ' ')
            self.data['starsTopo'].append(self.transform.transformERFA(ra, dec, 1))
            self.app.sharedMountDataLock.unlock()

    @PyQt5.QtCore.pyqtSlot()
    def handleReadyRead(self):
        # Get message from socket.
        while self.socket.bytesAvailable() and self.isRunning:
            self.messageString += self.socket.read(1024).decode()
        if self.data['FW'] < 21500:
            if len(self.messageString) < 18:
                return
            else:
                messageToProcess = self.messageString[:18]
                self.messageString = self.messageString[18:]
        else:
            if len(self.messageString) < 31:
                return
            else:
                messageToProcess = self.messageString[:31]
                self.messageString = self.messageString[31:]
        # Try and parse the message.
        try:
            if len(messageToProcess) == 0:
                return
            self.app.sharedMountDataLock.lockForWrite()
            valueList = messageToProcess.strip('#').split('#')
            #  +029.8# 1 0 1 +90# +00# V,2018-03-24#
            # all parameters are delivered
            if 2 < len(valueList) < 5:
                if len(valueList[0]) > 0:
                    self.data['TelescopeTempDEC'] = valueList[0]
                if len(valueList[1]) > 0:
                    self.data['RefractionStatus'] = valueList[1][0]
                    self.data['UnattendedFlip'] = valueList[1][1]
                    self.data['DualAxisTracking'] = valueList[1][2]
                    self.data['CurrentHorizonLimitHigh'] = valueList[1][3:]
                if len(valueList[2]) > 0:
                    self.data['CurrentHorizonLimitLow'] = valueList[2]
                if self.data['FW'] > 21500 and len(valueList[3]) > 0:
                    valid, expirationDate = valueList[3].split(',')
                    self.data['UTCDataValid'] = valid
                    self.data['UTCDataExpirationDate'] = expirationDate
            else:
                self.logger.warning('Parsing Status Slow combined command valueList is not OK: length:{0} content:{1}'.format(len(valueList), valueList))
        except Exception as e:
            self.logger.error('Problem parsing response, error: {0}, message:{1}'.format(e, messageToProcess))
        finally:
            self.app.sharedMountDataLock.unlock()
        self.sendLock = False



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'image_window_ui.ui'
#
# Created by: PyQt5 UI code generator 5.10.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ImageDialog(object):
    def setupUi(self, ImageDialog):
        ImageDialog.setObjectName("ImageDialog")
        ImageDialog.resize(791, 641)
        font = QtGui.QFont()
        font.setFamily("Arial")
        ImageDialog.setFont(font)
        self.image = QtWidgets.QWidget(ImageDialog)
        self.image.setGeometry(QtCore.QRect(5, 130, 781, 506))
        self.image.setAutoFillBackground(True)
        self.image.setObjectName("image")
        self.btn_expose = QtWidgets.QPushButton(ImageDialog)
        self.btn_expose.setGeometry(QtCore.QRect(15, 15, 76, 26))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_expose.sizePolicy().hasHeightForWidth())
        self.btn_expose.setSizePolicy(sizePolicy)
        self.btn_expose.setMinimumSize(QtCore.QSize(76, 0))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_expose.setFont(font)
        self.btn_expose.setObjectName("btn_expose")
        self.groupBox_2 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_2.setGeometry(QtCore.QRect(710, 10, 71, 106))
        self.groupBox_2.setObjectName("groupBox_2")
        self.btn_strechLow = QtWidgets.QRadioButton(self.groupBox_2)
        self.btn_strechLow.setGeometry(QtCore.QRect(10, 20, 61, 21))
        self.btn_strechLow.setObjectName("btn_strechLow")
        self.btn_strechMid = QtWidgets.QRadioButton(self.groupBox_2)
        self.btn_strechMid.setGeometry(QtCore.QRect(10, 40, 61, 21))
        self.btn_strechMid.setObjectName("btn_strechMid")
        self.btn_strechHigh = QtWidgets.QRadioButton(self.groupBox_2)
        self.btn_strechHigh.setGeometry(QtCore.QRect(10, 60, 61, 21))
        self.btn_strechHigh.setObjectName("btn_strechHigh")
        self.btn_strechSuper = QtWidgets.QRadioButton(self.groupBox_2)
        self.btn_strechSuper.setGeometry(QtCore.QRect(10, 80, 61, 21))
        self.btn_strechSuper.setObjectName("btn_strechSuper")
        self.groupBox_3 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_3.setGeometry(QtCore.QRect(630, 10, 76, 106))
        self.groupBox_3.setObjectName("groupBox_3")
        self.btn_size25 = QtWidgets.QRadioButton(self.groupBox_3)
        self.btn_size25.setGeometry(QtCore.QRect(10, 40, 61, 21))
        self.btn_size25.setObjectName("btn_size25")
        self.btn_size50 = QtWidgets.QRadioButton(self.groupBox_3)
        self.btn_size50.setGeometry(QtCore.QRect(10, 60, 61, 21))
        self.btn_size50.setObjectName("btn_size50")
        self.btn_size100 = QtWidgets.QRadioButton(self.groupBox_3)
        self.btn_size100.setGeometry(QtCore.QRect(10, 80, 61, 21))
        self.btn_size100.setObjectName("btn_size100")
        self.btn_size12 = QtWidgets.QRadioButton(self.groupBox_3)
        self.btn_size12.setGeometry(QtCore.QRect(10, 20, 61, 21))
        self.btn_size12.setObjectName("btn_size12")
        self.btn_solve = QtWidgets.QPushButton(ImageDialog)
        self.btn_solve.setGeometry(QtCore.QRect(175, 15, 76, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_solve.setFont(font)
        self.btn_solve.setObjectName("btn_solve")
        self.groupBox_4 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_4.setGeometry(QtCore.QRect(530, 10, 96, 106))
        self.groupBox_4.setObjectName("groupBox_4")
        self.btn_colorGrey = QtWidgets.QRadioButton(self.groupBox_4)
        self.btn_colorGrey.setGeometry(QtCore.QRect(10, 20, 71, 21))
        self.btn_colorGrey.setObjectName("btn_colorGrey")
        self.btn_colorCool = QtWidgets.QRadioButton(self.groupBox_4)
        self.btn_colorCool.setGeometry(QtCore.QRect(10, 40, 71, 21))
        self.btn_colorCool.setObjectName("btn_colorCool")
        self.btn_colorRainbow = QtWidgets.QRadioButton(self.groupBox_4)
        self.btn_colorRainbow.setGeometry(QtCore.QRect(10, 60, 81, 21))
        self.btn_colorRainbow.setObjectName("btn_colorRainbow")
        self.btn_colorSpectral = QtWidgets.QRadioButton(self.groupBox_4)
        self.btn_colorSpectral.setGeometry(QtCore.QRect(10, 80, 81, 21))
        self.btn_colorSpectral.setObjectName("btn_colorSpectral")
        self.imageBackground = QtWidgets.QLabel(ImageDialog)
        self.imageBackground.setGeometry(QtCore.QRect(0, 0, 790, 126))
        self.imageBackground.setText("")
        self.imageBackground.setObjectName("imageBackground")
        self.btn_loadFits = QtWidgets.QPushButton(ImageDialog)
        self.btn_loadFits.setEnabled(True)
        self.btn_loadFits.setGeometry(QtCore.QRect(95, 15, 76, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_loadFits.setFont(font)
        self.btn_loadFits.setObjectName("btn_loadFits")
        self.btn_cancel = QtWidgets.QPushButton(ImageDialog)
        self.btn_cancel.setGeometry(QtCore.QRect(15, 85, 76, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_cancel.setFont(font)
        self.btn_cancel.setObjectName("btn_cancel")
        self.le_imageFile = QtWidgets.QLineEdit(ImageDialog)
        self.le_imageFile.setEnabled(False)
        self.le_imageFile.setGeometry(QtCore.QRect(95, 50, 156, 26))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_imageFile.setFont(font)
        self.le_imageFile.setMouseTracking(False)
        self.le_imageFile.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_imageFile.setAcceptDrops(False)
        self.le_imageFile.setText("")
        self.le_imageFile.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_imageFile.setReadOnly(True)
        self.le_imageFile.setObjectName("le_imageFile")
        self.btn_exposeCont = QtWidgets.QPushButton(ImageDialog)
        self.btn_exposeCont.setGeometry(QtCore.QRect(15, 50, 76, 26))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_exposeCont.setFont(font)
        self.btn_exposeCont.setObjectName("btn_exposeCont")
        self.checkShowCrosshairs = QtWidgets.QCheckBox(ImageDialog)
        self.checkShowCrosshairs.setGeometry(QtCore.QRect(105, 90, 131, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkShowCrosshairs.setFont(font)
        self.checkShowCrosshairs.setChecked(False)
        self.checkShowCrosshairs.setObjectName("checkShowCrosshairs")
        self.imageMarker = QtWidgets.QWidget(ImageDialog)
        self.imageMarker.setGeometry(QtCore.QRect(5, 130, 781, 506))
        self.imageMarker.setAutoFillBackground(True)
        self.imageMarker.setObjectName("imageMarker")
        self.groupBox = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox.setGeometry(QtCore.QRect(260, 10, 141, 106))
        self.groupBox.setObjectName("groupBox")
        self.le_astrometrySolvingTime = QtWidgets.QLineEdit(self.groupBox)
        self.le_astrometrySolvingTime.setEnabled(False)
        self.le_astrometrySolvingTime.setGeometry(QtCore.QRect(95, 70, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_astrometrySolvingTime.setFont(font)
        self.le_astrometrySolvingTime.setMouseTracking(False)
        self.le_astrometrySolvingTime.setAcceptDrops(False)
        self.le_astrometrySolvingTime.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_astrometrySolvingTime.setText("")
        self.le_astrometrySolvingTime.setMaxLength(15)
        self.le_astrometrySolvingTime.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_astrometrySolvingTime.setReadOnly(False)
        self.le_astrometrySolvingTime.setObjectName("le_astrometrySolvingTime")
        self.le_astrometryStatusText = QtWidgets.QLineEdit(self.groupBox)
        self.le_astrometryStatusText.setEnabled(False)
        self.le_astrometryStatusText.setGeometry(QtCore.QRect(10, 70, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.le_astrometryStatusText.setFont(font)
        self.le_astrometryStatusText.setMouseTracking(False)
        self.le_astrometryStatusText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_astrometryStatusText.setAcceptDrops(False)
        self.le_astrometryStatusText.setText("")
        self.le_astrometryStatusText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_astrometryStatusText.setReadOnly(True)
        self.le_astrometryStatusText.setObjectName("le_astrometryStatusText")
        self.le_cameraExposureTime = QtWidgets.QLineEdit(self.groupBox)
        self.le_cameraExposureTime.setEnabled(False)
        self.le_cameraExposureTime.setGeometry(QtCore.QRect(95, 30, 36, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_cameraExposureTime.setFont(font)
        self.le_cameraExposureTime.setMouseTracking(False)
        self.le_cameraExposureTime.setAcceptDrops(False)
        self.le_cameraExposureTime.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_cameraExposureTime.setText("")
        self.le_cameraExposureTime.setMaxLength(15)
        self.le_cameraExposureTime.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_cameraExposureTime.setReadOnly(False)
        self.le_cameraExposureTime.setObjectName("le_cameraExposureTime")
        self.label_84 = QtWidgets.QLabel(self.groupBox)
        self.label_84.setGeometry(QtCore.QRect(10, 15, 81, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_84.setFont(font)
        self.label_84.setObjectName("label_84")
        self.label_83 = QtWidgets.QLabel(self.groupBox)
        self.label_83.setGeometry(QtCore.QRect(10, 55, 81, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_83.setFont(font)
        self.label_83.setObjectName("label_83")
        self.le_cameraStatusText = QtWidgets.QLineEdit(self.groupBox)
        self.le_cameraStatusText.setEnabled(False)
        self.le_cameraStatusText.setGeometry(QtCore.QRect(10, 30, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.le_cameraStatusText.setFont(font)
        self.le_cameraStatusText.setMouseTracking(False)
        self.le_cameraStatusText.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_cameraStatusText.setAcceptDrops(False)
        self.le_cameraStatusText.setText("")
        self.le_cameraStatusText.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.le_cameraStatusText.setReadOnly(True)
        self.le_cameraStatusText.setObjectName("le_cameraStatusText")
        self.groupBox_5 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_5.setGeometry(QtCore.QRect(405, 10, 120, 106))
        self.groupBox_5.setObjectName("groupBox_5")
        self.label_86 = QtWidgets.QLabel(self.groupBox_5)
        self.label_86.setGeometry(QtCore.QRect(10, 45, 26, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_86.setFont(font)
        self.label_86.setObjectName("label_86")
        self.le_AngleJ2000 = QtWidgets.QLineEdit(self.groupBox_5)
        self.le_AngleJ2000.setEnabled(False)
        self.le_AngleJ2000.setGeometry(QtCore.QRect(55, 70, 51, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_AngleJ2000.setFont(font)
        self.le_AngleJ2000.setMouseTracking(False)
        self.le_AngleJ2000.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_AngleJ2000.setAcceptDrops(False)
        self.le_AngleJ2000.setText("")
        self.le_AngleJ2000.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_AngleJ2000.setReadOnly(True)
        self.le_AngleJ2000.setObjectName("le_AngleJ2000")
        self.label_87 = QtWidgets.QLabel(self.groupBox_5)
        self.label_87.setGeometry(QtCore.QRect(10, 70, 36, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_87.setFont(font)
        self.label_87.setObjectName("label_87")
        self.le_RaJ2000 = QtWidgets.QLineEdit(self.groupBox_5)
        self.le_RaJ2000.setEnabled(False)
        self.le_RaJ2000.setGeometry(QtCore.QRect(40, 20, 66, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_RaJ2000.setFont(font)
        self.le_RaJ2000.setMouseTracking(False)
        self.le_RaJ2000.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_RaJ2000.setAcceptDrops(False)
        self.le_RaJ2000.setText("")
        self.le_RaJ2000.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_RaJ2000.setReadOnly(True)
        self.le_RaJ2000.setObjectName("le_RaJ2000")
        self.le_DecJ2000 = QtWidgets.QLineEdit(self.groupBox_5)
        self.le_DecJ2000.setEnabled(False)
        self.le_DecJ2000.setGeometry(QtCore.QRect(40, 45, 66, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.le_DecJ2000.setFont(font)
        self.le_DecJ2000.setMouseTracking(False)
        self.le_DecJ2000.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_DecJ2000.setAcceptDrops(False)
        self.le_DecJ2000.setText("")
        self.le_DecJ2000.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_DecJ2000.setReadOnly(True)
        self.le_DecJ2000.setObjectName("le_DecJ2000")
        self.label_85 = QtWidgets.QLabel(self.groupBox_5)
        self.label_85.setGeometry(QtCore.QRect(10, 20, 21, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_85.setFont(font)
        self.label_85.setObjectName("label_85")
        self.imageBackground.raise_()
        self.image.raise_()
        self.btn_expose.raise_()
        self.groupBox_2.raise_()
        self.groupBox_3.raise_()
        self.btn_solve.raise_()
        self.groupBox_4.raise_()
        self.btn_loadFits.raise_()
        self.btn_cancel.raise_()
        self.le_imageFile.raise_()
        self.btn_exposeCont.raise_()
        self.checkShowCrosshairs.raise_()
        self.imageMarker.raise_()
        self.groupBox.raise_()
        self.groupBox_5.raise_()

        self.retranslateUi(ImageDialog)
        QtCore.QMetaObject.connectSlotsByName(ImageDialog)

    def retranslateUi(self, ImageDialog):
        _translate = QtCore.QCoreApplication.translate
        ImageDialog.setWindowTitle(_translate("ImageDialog", "Imaging"))
        self.btn_expose.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Single exposure</p></body></html>"))
        self.btn_expose.setText(_translate("ImageDialog", "Expose 1"))
        self.groupBox_2.setTitle(_translate("ImageDialog", "Strech"))
        self.btn_strechLow.setText(_translate("ImageDialog", "Low"))
        self.btn_strechMid.setText(_translate("ImageDialog", "Mid"))
        self.btn_strechHigh.setText(_translate("ImageDialog", "High"))
        self.btn_strechSuper.setText(_translate("ImageDialog", "Super"))
        self.groupBox_3.setTitle(_translate("ImageDialog", "Zoom"))
        self.btn_size25.setText(_translate("ImageDialog", "4x"))
        self.btn_size50.setText(_translate("ImageDialog", "2x"))
        self.btn_size100.setText(_translate("ImageDialog", "1x"))
        self.btn_size12.setText(_translate("ImageDialog", "8x"))
        self.btn_solve.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Single plate solve of the actual image</p></body></html>"))
        self.btn_solve.setText(_translate("ImageDialog", "Solve"))
        self.groupBox_4.setTitle(_translate("ImageDialog", "Colors"))
        self.btn_colorGrey.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Color scheme black /white</p></body></html>"))
        self.btn_colorGrey.setText(_translate("ImageDialog", "Grey"))
        self.btn_colorCool.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Color scheme red/blue</p></body></html>"))
        self.btn_colorCool.setText(_translate("ImageDialog", "Cool"))
        self.btn_colorRainbow.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Color scheme rainbow</p></body></html>"))
        self.btn_colorRainbow.setText(_translate("ImageDialog", "Rainbow"))
        self.btn_colorSpectral.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Color scheme rainbow</p></body></html>"))
        self.btn_colorSpectral.setText(_translate("ImageDialog", "Spectral"))
        self.imageBackground.setProperty("color", _translate("ImageDialog", "blue"))
        self.btn_loadFits.setToolTip(_translate("ImageDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Load a fits file and display is</span></p></body></html>"))
        self.btn_loadFits.setText(_translate("ImageDialog", "Load FITS"))
        self.btn_cancel.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Cancels an imaging or plate solving action or stops continous exposures</p></body></html>"))
        self.btn_cancel.setText(_translate("ImageDialog", "Cancel"))
        self.le_imageFile.setToolTip(_translate("ImageDialog", "<html><head/><body><p>name of image which is shown</p></body></html>"))
        self.btn_exposeCont.setToolTip(_translate("ImageDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Continous exposures</span></p></body></html>"))
        self.btn_exposeCont.setText(_translate("ImageDialog", "Expose N"))
        self.checkShowCrosshairs.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Show crosshairs on image</p></body></html>"))
        self.checkShowCrosshairs.setText(_translate("ImageDialog", "Show crosshairs"))
        self.groupBox.setTitle(_translate("ImageDialog", "Device Stats"))
        self.le_astrometrySolvingTime.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Time elapsed for plate solving</p></body></html>"))
        self.le_astrometryStatusText.setToolTip(_translate("ImageDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Status feedback from astrometry</span></p></body></html>"))
        self.le_cameraExposureTime.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Time left for image integration</p></body></html>"))
        self.label_84.setText(_translate("ImageDialog", "Camera"))
        self.label_83.setText(_translate("ImageDialog", "Astrometry"))
        self.le_cameraStatusText.setToolTip(_translate("ImageDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Status feedback from camera</span></p></body></html>"))
        self.groupBox_5.setTitle(_translate("ImageDialog", "Solve Result"))
        self.label_86.setText(_translate("ImageDialog", "DEC"))
        self.le_AngleJ2000.setToolTip(_translate("ImageDialog", "<html><head/><body><p>Shows the solved angle of image in degrees</p></body></html>"))
        self.label_87.setText(_translate("ImageDialog", "Angle"))
        self.le_RaJ2000.setToolTip(_translate("ImageDialog", "<html><head/><body><p><span style=\" font-weight:400;\">Shows the solved RA of image in J2000 coordinates</span></p></body></html>"))
        self.le_DecJ2000.setToolTip(_translate("ImageDialog", "<html><head/><body><p><span style=\" font-weight:400;\">Shows the solved DEC of image in J2000 coordinates</span></p></body></html>"))
        self.label_85.setText(_translate("ImageDialog", "RA"))




############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.5
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import time
import PyQt5
import requests
from requests_toolbelt.multipart import encoder
from baseclasses import checkIP
import json
import collections
import copy


class AstrometryClient:
    logger = logging.getLogger(__name__)

    solveData = {'session': '12345',
                 'allow_commercial_use': 'd',
                 'allow_modifications': 'd',
                 'publicly_visible': 'n',
                 'scale_units': 'arcsecperpix',
                 'scale_type': 'ev',
                 'scale_est': 1.3,
                 'scale_err': 20,
                 'center_ra': 315,
                 'center_dec': 68,
                 'radius': 1,
                 'downsample_factor': 2,
                 'use_sextractor': False,
                 'crpix_center': True,
                 'parity': 2
                 }

    def __init__(self, main, app, data):
        self.main = main
        self.app = app
        self.data = data
        self.application = dict()
        self.cancel = False
        self.mutexCancel = PyQt5.QtCore.QMutex()

        self.checkIP = checkIP.CheckIP()

        self.application = {
            'AstrometryHost': '192.168.2.161',
            'AstrometryPort': 3499,
            'URLLogin': '',
            'URLAPI': '',
            'APIKey': '',
            'TimeoutMax': 60,
            'Connected': False,
            'Available': True,
            'Name': 'ASTROMETRY.NET',
            'Status': ''
        }

        self.app.ui.le_AstrometryHost.editingFinished.connect(self.changeIPSettings)
        self.app.ui.le_AstrometryPort.editingFinished.connect(self.changeIPSettings)
        self.app.ui.le_AstrometryAPIKey.editingFinished.connect(self.changeIPSettings)

    def initConfig(self):
        try:
            if 'AstrometryTimeout' in self.app.config:
                self.app.ui.le_astrometryTimeout.setText(self.app.config['AstrometryTimeout'])
            if 'AstrometryHost' in self.app.config:
                self.app.ui.le_AstrometryHost.setText(self.app.config['AstrometryHost'])
            if 'AstrometryPort' in self.app.config:
                self.app.ui.le_AstrometryPort.setText(self.app.config['AstrometryPort'])
            if 'AstrometryAPIKey' in self.app.config:
                self.app.ui.le_AstrometryAPIKey.setText(self.app.config['AstrometryAPIKey'])
            if 'AstrometryDownsample' in self.app.config:
                self.app.ui.astrometryDownsampling.setValue(self.app.config['AstrometryDownsample'])
            if 'AstrometryRadius' in self.app.config:
                self.app.ui.astrometryRadius.setValue(self.app.config['AstrometryRadius'])
        except Exception as e:
            self.logger.error('Item in config.cfg for astrometry client could not be initialized, error:{0}'.format(e))
        finally:
            pass
        self.changeIPSettings()

    def storeConfig(self):
        self.app.config['AstrometryPort'] = self.app.ui.le_AstrometryPort.text()
        self.app.config['AstrometryHost'] = self.app.ui.le_AstrometryHost.text()
        self.app.config['AstrometryAPIKey'] = self.app.ui.le_AstrometryAPIKey.text()
        self.app.config['AstrometryTimeout'] = self.app.ui.le_astrometryTimeout.text()
        self.app.config['AstrometryDownsample'] = self.app.ui.astrometryDownsampling.value()
        self.app.config['AstrometryRadius'] = self.app.ui.astrometryRadius.value()

    def start(self):
        pass

    def stop(self):
        pass

    def setCancelAstrometry(self):
        self.mutexCancel.lock()
        self.cancel = True
        self.mutexCancel.unlock()

    def changeIPSettings(self):
        self.data['Status'] = 'ERROR'
        self.data['CONNECTION']['CONNECT'] = 'Off'
        host = self.app.ui.le_AstrometryHost.text()
        port = self.app.ui.le_AstrometryPort.text()
        self.application['AstrometryHost'] = host
        self.application['AstrometryPort'] = int(port)
        self.application['URLAPI'] = 'http://' + host + ':' + port + '/api'
        self.application['URLLogin'] = 'http://' + host + ':' + port + '/api/login'
        self.application['APIKey'] = self.app.ui.le_AstrometryAPIKey.text()
        self.application['Name'] = 'Astrometry'
        self.application['TimeoutMax'] = float(self.app.ui.le_astrometryTimeout.text())
        self.app.messageQueue.put('Setting IP address for astrometry to: {0}:{1}\n'.format(self.application['AstrometryHost'],
                                                                                           self.application['AstrometryPort']))
        self.logger.info('Setting IP address for astrometry to: {0}:{1}, key: {2}'.format(self.application['AstrometryHost'],
                                                                                          self.application['AstrometryPort'],
                                                                                          self.application['APIKey']))

    def getStatus(self):
        if self.application['URLAPI'] == '':
            return
        if self.checkIP.checkIPAvailable(self.application['AstrometryHost'], self.application['AstrometryPort']):
            self.application['Status'] = 'OK'
            self.data['CONNECTION']['CONNECT'] = 'On'
        else:
            self.data['Status'] = 'ERROR'
            self.data['CONNECTION']['CONNECT'] = 'Off'

    def callbackUpload(self, monitor):
        self.main.astrometrySolvingTime.emit('{0:3d}%'.format(int(monitor.bytes_read / monitor.len * 100)))

    def solveImage(self, imageParams):
        self.mutexCancel.lock()
        self.cancel = False
        self.mutexCancel.unlock()

        downsampleFactor = self.app.ui.astrometryDownsampling.value()
        radius = self.app.ui.astrometryRadius.value()
        # waiting for start solving
        timeSolvingStart = time.time()
        # defining start values
        errorState = False
        result = ''
        response = ''
        stat = ''
        submissionID = ''
        jobID = ''
        headers = dict()
        imageParams['Message'] = ''

        self.main.astrometryStatusText.emit('START')
        # check if we have the online solver running
        self.main.astrometrySolvingTime.emit('{0:02.0f}'.format(time.time()-timeSolvingStart))
        if self.application['APIKey'] != '':
            # we have to login with the api key for the online solver to get the session key
            try:
                response = requests.post(self.application['URLLogin'],
                                         data={'request-json': json.dumps({"apikey": self.application['APIKey']})},
                                         headers={})
                result = json.loads(response.text)
            except Exception as e:
                self.logger.error('Problem setting api key, error: {0}, result: {1}, response: {2}'
                                  .format(e, result, response))
                imageParams['Message'] = 'Login with api key failed'
                errorState = True
            finally:
                pass
            if not errorState:
                if 'status' in result:
                    if result['status'] == 'error':
                        self.app.messageQueue.put('Get session key for ASTROMETRY.NET failed because: {0}\n'.format(result['errormessage']))
                        self.logger.error('Get session key failed because: {0}'.format(result['errormessage']))
                        errorState = True
                    elif result['status'] == 'success':
                        self.solveData['session'] = result['session']
                        self.app.messageQueue.put('\tSession key for ASTROMETRY.NET is [{0}]\n'.format(result['session']))
                else:
                    imageParams['Message'] = 'Malformed result in login procedure'
                    errorState = True
        else:
            # local solve runs with dummy session key
            self.solveData['session'] = '12345'

        self.main.astrometrySolvingTime.emit('{0:02.0f}'.format(time.time()-timeSolvingStart))

        # loop for upload
        self.main.astrometryStatusText.emit('UPLOAD')
        # start uploading the data and define the parameters
        data = copy.copy(self.solveData)
        data['downsample_factor'] = downsampleFactor
        # check if you want to use this parameter. if 0, than remove it
        if radius > 0:
            data['radius'] = radius
        else:
            if 'radius' in data:
                del data['radius']
        data['scale_est'] = float(imageParams['ScaleHint'])
        # ra is in hours
        data['center_ra'] = imageParams['RaJ2000'] * 360 / 24
        data['center_dec'] = float(imageParams['DecJ2000'])

        if not errorState:
            fields = collections.OrderedDict()
            fields['request-json'] = json.dumps(data)
            fields['file'] = (imageParams['Imagepath'], open(imageParams['Imagepath'], 'rb'), 'application/octet-stream')
            encodedMultipart = encoder.MultipartEncoder(fields)
            monitorMultipart = encoder.MultipartEncoderMonitor(encodedMultipart, self.callbackUpload)
            try:
                result = ''
                response = requests.post(self.application['URLAPI'] + '/upload',
                                         data=monitorMultipart,
                                         headers={'Content-Type': monitorMultipart.content_type})
                result = json.loads(response.text)
                stat = result['status']
                self.logger.info('Result upload: {0}, reply: {1}'.format(result, response))
            except Exception as e:
                self.logger.error('Problem upload, error: {0}, result: {1}, response: {2}'.format(e, result, response))
                errorState = True
                imageParams['Message'] = 'Error upload'
            finally:
                pass
            if not errorState:
                if stat != 'success':
                    self.logger.warning('Could not upload image to astrometry server, error: {0}'.format(result))
                    imageParams['Message'] = 'Upload failed'
                    errorState = True
                else:
                    submissionID = result['subid']
        self.main.astrometrySolvingTime.emit('{0:02.0f}'.format(time.time()-timeSolvingStart))

        # loop for solve
        self.main.astrometryStatusText.emit('SOLVE-Sub')
        # wait for the submission = star detection algorithm to take place
        while not self.cancel and not errorState:
            data = {'request-json': ''}
            headers = {}
            try:
                result = ''
                response = requests.get(self.application['URLAPI'] + '/submissions/{0}'
                                        .format(submissionID),
                                        data=data,
                                        headers=headers)
                result = json.loads(response.text)
                self.logger.info('Result submissions: {0}, reply: {1}'.format(result, response))
            except Exception as e:
                self.logger.error('Problem submissions, error: {0}, result: {1}, response: {2}'
                                  .format(e, result, response))
                errorState = True
                imageParams['Message'] = 'Error submissions'
                break
            finally:
                pass
            if 'jobs' in result:
                jobs = result['jobs']
            else:
                self.logger.error('Problem submissions, job not found, result: {0}, response: {1}'.format(result, response))
                errorState = True
                break
            if len(jobs) > 0:
                if jobs[0] is not None:
                    jobID = jobs[0]
                    break
            if time.time()-timeSolvingStart > self.application['TimeoutMax']:
                # timeout after timeoutMax seconds
                errorState = True
                imageParams['Message'] = 'Timeout'
                break
            self.main.astrometrySolvingTime.emit('{0:02.0f}'.format(time.time()-timeSolvingStart))
            time.sleep(1)

        # waiting for the solving results done by jobs are present
        self.main.astrometryStatusText.emit('SOLVE-Job')
        while not self.cancel and not errorState:
            data = {'request-json': ''}
            headers = {}
            try:
                result = ''
                response = requests.get(self.application['URLAPI'] + '/jobs/{0}'
                                        .format(jobID),
                                        data=data,
                                        headers=headers)
                result = json.loads(response.text)
                self.logger.info('Result jobs: {0}, reply: {1}'.format(result, response))
            except Exception as e:
                self.logger.error('Problem jobs, error: {0}, result: {1}, response: {2}'.format(e, result, response))
                errorState = True
                imageParams['Message'] = 'Error jobs'
            finally:
                pass
            if 'status' in result:
                stat = result['status']
            else:
                self.logger.error('Problem jobs, status not found, result: {0}, response: {1}'.format(result, response))
                errorState = True
                break
            if stat == 'success':
                break
            if stat == 'failure':
                errorState = True
                break
            if time.time()-timeSolvingStart > self.application['TimeoutMax']:
                # timeout after timeoutMax seconds
                errorState = True
                imageParams['Message'] = 'Timeout'
                break
            self.main.astrometrySolvingTime.emit('{0:02.0f}'.format(time.time()-timeSolvingStart))
            time.sleep(1)

        # Loop for data
        self.main.imageSolved.emit()
        self.main.astrometryStatusText.emit('GET DATA')
        # now get the solving data and results
        if not self.cancel and not errorState:
            try:
                result = ''
                response = requests.get(self.application['URLAPI'] + '/jobs/{0}/calibration'
                                        .format(jobID),
                                        data=data,
                                        headers=headers)
                result = json.loads(response.text)
                self.logger.info('Result calibration: {0}, reply: {1}'.format(result, response))
                imageParams['Solved'] = True
                imageParams['RaJ2000Solved'] = result['ra'] * 24 / 360
                imageParams['DecJ2000Solved'] = result['dec']
                imageParams['Scale'] = result['pixscale']
                imageParams['Angle'] = result['orientation']
                imageParams['TimeTS'] = time.time()-timeSolvingStart
                self.main.astrometrySolvingTime.emit('{0:02.0f}'.format(time.time()-timeSolvingStart))
                imageParams['Message'] = 'Solved with success'
            except Exception as e:
                self.logger.error('Problem get calibration data, error: {0}, result: {1}, response: {2}'.format(e, result, response))
                imageParams['RaJ2000Solved'] = 0
                imageParams['DecJ2000Solved'] = 0
                imageParams['Scale'] = 0
                imageParams['Angle'] = 0
                imageParams['TimeTS'] = time.time()-timeSolvingStart
                imageParams['Solved'] = False
                imageParams['Message'] = 'Solve failed'
            finally:
                pass
        else:
            imageParams['Solved'] = False
            imageParams['Message'] = 'Solve failed'

        # finally idle
        self.main.imageDataDownloaded.emit()
        self.main.astrometryStatusText.emit('IDLE')
        self.main.astrometrySolvingTime.emit('')



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'message_window_ui.ui'
#
# Created by: PyQt5 UI code generator 5.10
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MessageDialog(object):
    def setupUi(self, MessageDialog):
        MessageDialog.setObjectName("MessageDialog")
        MessageDialog.resize(791, 641)
        font = QtGui.QFont()
        font.setFamily("Arial")
        MessageDialog.setFont(font)
        self.messages = QtWidgets.QTextBrowser(MessageDialog)
        self.messages.setGeometry(QtCore.QRect(5, 5, 781, 631))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.messages.sizePolicy().hasHeightForWidth())
        self.messages.setSizePolicy(sizePolicy)
        self.messages.setMinimumSize(QtCore.QSize(0, 100))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.messages.setFont(font)
        self.messages.setFrameShadow(QtWidgets.QFrame.Plain)
        self.messages.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.messages.setAcceptRichText(False)
        self.messages.setObjectName("messages")

        self.retranslateUi(MessageDialog)
        QtCore.QMetaObject.connectSlotsByName(MessageDialog)

    def retranslateUi(self, MessageDialog):
        _translate = QtCore.QCoreApplication.translate
        MessageDialog.setWindowTitle(_translate("MessageDialog", "Messages"))
        self.messages.setToolTip(_translate("MessageDialog", "Error Messages from Tool"))
        self.messages.setHtml(_translate("MessageDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Arial\'; font-size:10pt; font-weight:600; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'MS Shell Dlg 2\'; font-size:8pt; font-weight:400;\"><br /></p></body></html>"))




############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import os
import time
import numpy
import copy
import PyQt5
import astropy.io.fits as pyfits
from astropy.visualization import MinMaxInterval, ImageNormalize, AsymmetricPercentileInterval, PowerStretch
from matplotlib import use
from baseclasses import widget
from astrometry import transform
from gui import image_window_ui
use('Qt5Agg')


class WorkerSignals(PyQt5.QtCore.QObject):

    finished = PyQt5.QtCore.pyqtSignal()
    error = PyQt5.QtCore.pyqtSignal(object)
    result = PyQt5.QtCore.pyqtSignal(object)


class Worker(PyQt5.QtCore.QRunnable):

    def __init__(self, fn, *args, **kwargs):
        super(Worker, self).__init__()
        # Store constructor arguments (re-used for processing)
        self.fn = fn
        self.args = args
        self.kwargs = kwargs
        self.signals = WorkerSignals()

    @PyQt5.QtCore.pyqtSlot()
    def run(self):
        try:
            result = self.fn(*self.args, **self.kwargs)
        except Exception as e:
            self.signals.error.emit(e)
            print(e)
        else:
            self.signals.result.emit(result)
        finally:
            self.signals.finished.emit()


class ImagesWindow(widget.MwWidget):
    logger = logging.getLogger(__name__)
    BASENAME = 'exposure-'
    signalShowFitsImage = PyQt5.QtCore.pyqtSignal(str)
    signalSolveFitsImage = PyQt5.QtCore.pyqtSignal(str)
    signalSetRaSolved = PyQt5.QtCore.pyqtSignal(str)
    signalSetDecSolved = PyQt5.QtCore.pyqtSignal(str)
    signalSetAngleSolved = PyQt5.QtCore.pyqtSignal(str)
    signalSetManualEnable = PyQt5.QtCore.pyqtSignal(bool)
    signalDisplayImage = PyQt5.QtCore.pyqtSignal(object)

    def __init__(self, app):
        super(ImagesWindow, self).__init__()
        self.app = app
        self.showStatus = False
        self.cancel = False
        self.imagePath = ''
        self.imageReady = False
        self.solveReady = False
        self.transform = transform.Transform(self.app)
        self.ui = image_window_ui.Ui_ImageDialog()
        self.ui.setupUi(self)
        self.initUI()
        # allow sizing of the window
        self.setFixedSize(PyQt5.QtCore.QSize(16777215, 16777215))
        # set the minimum size
        self.setMinimumSize(791, 400)
        self.image = numpy.zeros([20, 20])
        self.ui.btn_strechLow.setChecked(True)
        self.ui.btn_size100.setChecked(True)
        self.ui.btn_colorGrey.setChecked(True)
        self.threadpool = PyQt5.QtCore.QThreadPool()

        # adding the matplotlib integration
        self.imageMatplotlib = widget.IntegrateMatplotlib(self.ui.image)
        # making background looking transparent
        self.imageMatplotlib.fig.patch.set_facecolor('none')
        background = self.imageMatplotlib.fig.canvas.parentWidget()
        background.setStyleSheet('background-color: transparent;')
        self.imageMatplotlib.axes = self.imageMatplotlib.fig.add_subplot(111)
        self.imageMatplotlib.axes.set_axis_off()
        self.imageMatplotlib.fig.subplots_adjust(left=0.01, right=0.99, bottom=0.01, top=0.99)

        self.imageMatplotlibMarker = widget.IntegrateMatplotlib(self.ui.imageMarker)
        # making background looking transparent
        self.imageMatplotlibMarker.fig.patch.set_facecolor('none')
        background = self.imageMatplotlibMarker.fig.canvas.parentWidget()
        background.setStyleSheet('background-color: transparent;')
        self.imageMatplotlibMarker.axes = self.imageMatplotlibMarker.fig.add_subplot(111)
        self.imageMatplotlibMarker.axes.set_axis_off()
        self.imageMatplotlibMarker.fig.subplots_adjust(left=0.01, right=0.99, bottom=0.01, top=0.99)

        # slots for gui elements
        self.ui.btn_expose.clicked.connect(self.exposeOnce)
        self.ui.btn_exposeCont.clicked.connect(self.exposeCont)
        self.ui.btn_solve.clicked.connect(self.solveOnce)
        self.ui.btn_colorGrey.clicked.connect(self.setColor)
        self.ui.btn_colorCool.clicked.connect(self.setColor)
        self.ui.btn_colorRainbow.clicked.connect(self.setColor)
        self.ui.btn_colorSpectral.clicked.connect(self.setColor)
        self.ui.btn_size12.clicked.connect(self.setZoom)
        self.ui.btn_size25.clicked.connect(self.setZoom)
        self.ui.btn_size50.clicked.connect(self.setZoom)
        self.ui.btn_size100.clicked.connect(self.setZoom)
        self.ui.btn_strechLow.clicked.connect(self.setStrech)
        self.ui.btn_strechMid.clicked.connect(self.setStrech)
        self.ui.btn_strechHigh.clicked.connect(self.setStrech)
        self.ui.btn_strechSuper.clicked.connect(self.setStrech)
        self.ui.btn_cancel.clicked.connect(self.cancelAction)
        self.ui.checkShowCrosshairs.stateChanged.connect(self.setCrosshairOnOff)

        # define the slots for signals
        self.signalShowFitsImage.connect(self.showFitsImage)
        self.signalSolveFitsImage.connect(self.solveFitsImage)
        self.signalSetRaSolved.connect(self.setRaSolved)
        self.signalSetDecSolved.connect(self.setDecSolved)
        self.signalSetAngleSolved.connect(self.setAngleSolved)
        self.ui.btn_loadFits.clicked.connect(self.loadFitsFileFrom)
        self.app.workerImaging.imageSaved.connect(self.setImageReady)
        self.app.workerAstrometry.imageDataDownloaded.connect(self.setSolveReady)
        self.signalSetManualEnable.connect(self.setManualEnable)
        self.signalDisplayImage.connect(self.displayImage)

    def resizeEvent(self, QResizeEvent):
        # allow message window to be resized in height
        self.ui.image.setGeometry(5, 125, self.width() - 10, self.height() - 125)
        self.ui.imageMarker.setGeometry(5, 125, self.width() - 10, self.height() - 125)
        # getting position of axis
        axesPos = self.imageMatplotlibMarker.axes.get_position()
        # and using it fo the other plot widgets to be identically same size and position
        self.imageMatplotlib.axes.set_position(axesPos)
        # size the header window as well
        self.ui.imageBackground.setGeometry(0, 0, self.width(), 126)

    def initConfig(self):
        try:
            if 'ImagePopupWindowPositionX' in self.app.config:
                x = self.app.config['ImagePopupWindowPositionX']
                y = self.app.config['ImagePopupWindowPositionY']
                if x > self.screenSizeX:
                    x = 0
                if y > self.screenSizeY:
                    y = 0
                self.move(x, y)
            if 'ImagePopupWindowShowStatus' in self.app.config:
                self.showStatus = self.app.config['ImagePopupWindowShowStatus']
            if 'ImageWindowHeight' in self.app.config and 'ImageWindowWidth' in self.app.config:
                self.resize(self.app.config['ImageWindowWidth'], self.app.config['ImageWindowHeight'])
            if 'ImagePath' in self.app.config:
                self.imagePath = self.app.config['ImagePath']
                self.ui.le_imageFile.setText(os.path.basename(self.imagePath))
            if 'CheckShowCrosshairs' in self.app.config:
                self.ui.checkShowCrosshairs.setChecked(self.app.config['CheckShowCrosshairs'])
            if 'ColorCool' in self.app.config:
                self.ui.btn_colorCool.setChecked(self.app.config['ColorCool'])
            if 'ColorRainbow' in self.app.config:
                self.ui.btn_colorRainbow.setChecked(self.app.config['ColorRainbow'])
            if 'ColorSpectral' in self.app.config:
                self.ui.btn_colorSpectral.setChecked(self.app.config['ColorSpectral'])
            if 'ColorGrey' in self.app.config:
                self.ui.btn_colorGrey.setChecked(self.app.config['ColorGrey'])
            if 'Size12' in self.app.config:
                self.ui.btn_size12.setChecked(self.app.config['Size12'])
            if 'Size25' in self.app.config:
                self.ui.btn_size25.setChecked(self.app.config['Size25'])
            if 'Size50' in self.app.config:
                self.ui.btn_size50.setChecked(self.app.config['Size50'])
            if 'Size100' in self.app.config:
                self.ui.btn_size100.setChecked(self.app.config['Size100'])
            if 'StrechLow' in self.app.config:
                self.ui.btn_strechLow.setChecked(self.app.config['StrechLow'])
            if 'StrechMid' in self.app.config:
                self.ui.btn_strechMid.setChecked(self.app.config['StrechMid'])
            if 'StrechHigh' in self.app.config:
                self.ui.btn_strechHigh.setChecked(self.app.config['StrechHigh'])
            if 'StrechSuper' in self.app.config:
                self.ui.btn_strechSuper.setChecked(self.app.config['StrechSuper'])

        except Exception as e:
            self.logger.error('Item in config.cfg not be initialized for image window, error:{0}'.format(e))
        finally:
            pass
        self.setCrosshairOnOff()

    def storeConfig(self):
        self.app.config['ImagePopupWindowPositionX'] = self.pos().x()
        self.app.config['ImagePopupWindowPositionY'] = self.pos().y()
        self.app.config['ImagePopupWindowShowStatus'] = self.showStatus
        self.app.config['ColorCool'] = self.ui.btn_colorCool.isChecked()
        self.app.config['ColorRainbow'] = self.ui.btn_colorRainbow.isChecked()
        self.app.config['ColorSpectral'] = self.ui.btn_colorSpectral.isChecked()
        self.app.config['ColorGrey'] = self.ui.btn_colorGrey.isChecked()
        self.app.config['Size12'] = self.ui.btn_size12.isChecked()
        self.app.config['Size25'] = self.ui.btn_size25.isChecked()
        self.app.config['Size50'] = self.ui.btn_size50.isChecked()
        self.app.config['Size100'] = self.ui.btn_size100.isChecked()
        self.app.config['StrechLow'] = self.ui.btn_strechLow.isChecked()
        self.app.config['StrechMid'] = self.ui.btn_strechMid.isChecked()
        self.app.config['StrechHigh'] = self.ui.btn_strechHigh.isChecked()
        self.app.config['StrechSuper'] = self.ui.btn_strechSuper.isChecked()
        self.app.config['ImagePath'] = self.imagePath
        self.app.config['CheckShowCrosshairs'] = self.ui.checkShowCrosshairs.isChecked()
        self.app.config['ImageWindowHeight'] = self.height()
        self.app.config['ImageWindowWidth'] = self.width()

    def toggleWindow(self):
        self.showStatus = not self.showStatus
        if self.showStatus:
            self.showWindow()
        else:
            self.close()

    def showWindow(self):
        self.showStatus = True
        self.signalShowFitsImage.emit(self.imagePath)
        self.app.signalChangeStylesheet.emit(self.app.ui.btn_openImageWindow, 'running', 'true')
        self.setVisible(True)

    def closeEvent(self, closeEvent):
        super().closeEvent(closeEvent)
        self.app.signalChangeStylesheet.emit(self.app.ui.btn_openImageWindow, 'running', 'false')

    def cancelAction(self):
        self.cancel = True
        self.app.workerAstrometry.astrometryCancel.emit()
        self.app.workerImaging.imagingCancel.emit()

    def setRaSolved(self, text):
        self.ui.le_RaJ2000.setText(text)

    def setDecSolved(self, text):
        self.ui.le_DecJ2000.setText(text)

    def setAngleSolved(self, text):
        self.ui.le_AngleJ2000.setText(text)

    def setImageReady(self):
        self.imageReady = True

    def setSolveReady(self):
        self.solveReady = True

    def loadFitsFileFrom(self):
        value, ext = self.selectFile(self, 'Open FITS file', '/images', 'FITS files (*.fit*)', True)
        if value != '':
            self.signalShowFitsImage.emit(value + ext)
        else:
            self.logger.warning('No Fits file file selected')

    def drawMarkers(self):
        # fixed points and horizon plane
        self.ui.image.stackUnder(self.ui.imageMarker)
        self.imageMatplotlibMarker.axes.cla()
        self.imageMatplotlibMarker.axes.set_xlim(0, 400)
        self.imageMatplotlibMarker.axes.set_ylim(0, 400)
        self.imageMatplotlibMarker.axes.grid(True, color='#606060', ls='dotted')
        self.imageMatplotlibMarker.axes.spines['bottom'].set_color((0, 0, 0, 0))
        self.imageMatplotlibMarker.axes.spines['top'].set_color((0, 0, 0, 0))
        self.imageMatplotlibMarker.axes.spines['left'].set_color((0, 0, 0, 0))
        self.imageMatplotlibMarker.axes.spines['right'].set_color((0, 0, 0, 0))
        self.imageMatplotlibMarker.axes.set_facecolor((0, 0, 0, 0))
        self.imageMatplotlibMarker.axes.set_xticks(numpy.arange(20, 381, 40))
        self.imageMatplotlibMarker.axes.set_yticks(numpy.arange(20, 381, 40))
        self.imageMatplotlibMarker.axes.set_xticklabels([])
        self.imageMatplotlibMarker.axes.set_yticklabels([])
        self.imageMatplotlibMarker.axes.plot(200, 200, zorder=10, color='#606060', marker='o', markersize=25, markeredgewidth=2, fillstyle='none')
        self.imageMatplotlibMarker.axes.plot(200, 200, zorder=10, color='#606060', marker='o', markersize=10, markeredgewidth=1, fillstyle='none')
        self.imageMatplotlibMarker.fig.canvas.draw()

    @PyQt5.QtCore.pyqtSlot(str)
    def showFitsImage(self, filename):
        # fits file ahs to be there
        if not os.path.isfile(filename):
            return
        # image window has to be present
        if not self.showStatus:
            return
        self.signalSetRaSolved.emit('')
        self.signalSetDecSolved.emit('')
        self.signalSetAngleSolved.emit('')
        self.imagePath = filename
        self.ui.le_imageFile.setText(os.path.basename(self.imagePath))
        try:
            fitsFileHandle = pyfits.open(filename)
            error = False
        except Exception as e:
            error = True
            if fitsFileHandle:
                fitsFileHandle.close()
            self.logger.error('File {0} could not be loaded, error: {1}'.format(self.imagePath, e))
        finally:
            if error:
                return
        self.image = copy.copy(fitsFileHandle[0].data)
        fitsFileHandle.close()
        strechMode = self.getStrechMode()
        colorMode = self.getColorMode()
        zoomMode = self.getZoomMode()
        worker = Worker(self.calculateImage, self.image, strechMode, colorMode, zoomMode)
        worker.signals.result.connect(self.signalDisplayImage)
        self.threadpool.start(worker)

    @PyQt5.QtCore.pyqtSlot(object)
    def displayImage(self, result):
        image = result[0]
        color = result[1]
        norm = result[2]
        self.imageMatplotlib.axes.imshow(image, cmap=color, norm=norm)
        self.imageMatplotlib.fig.canvas.draw()
        self.drawMarkers()
        self.resizeEvent(0)

    @staticmethod
    def calculateImage(imageOrig, strechMode, colorMode, zoomMode):
        sizeX, sizeY = imageOrig.shape
        # calculate the cropping parameters
        if zoomMode == 12:
            minx = int(sizeX * 7 / 16)
            maxx = minx + int(sizeX / 8)
            miny = int(sizeY * 7 / 16)
            maxy = miny + int(sizeY / 8)
        elif zoomMode == 25:
            minx = int(sizeX * 3 / 8)
            maxx = minx + int(sizeX / 4)
            miny = int(sizeY * 3 / 8)
            maxy = miny + int(sizeY / 4)
        elif zoomMode == 50:
            minx = int(sizeX / 4)
            maxx = minx + int(sizeX / 2)
            miny = int(sizeY / 4)
            maxy = miny + int(sizeY / 2)
        else:
            minx = 0
            maxx = sizeX
            miny = 0
            maxy = sizeY
        # crop image
        image = imageOrig[minx:maxx, miny:maxy]
        # calculation the strech
        if strechMode == 'Low':
            interval = AsymmetricPercentileInterval(98, 99.998)
        elif strechMode == 'Mid':
            interval = AsymmetricPercentileInterval(25, 99.95)
        elif strechMode == 'High':
            interval = AsymmetricPercentileInterval(12, 99.9)
        else:
            interval = AsymmetricPercentileInterval(1, 99.8)
        vmin, vmax = interval.get_limits(image)
        # Create an ImageNormalize object using a LogStrech object
        norm = ImageNormalize(vmin=vmin, vmax=vmax, stretch=PowerStretch(1))
        result = (image, colorMode, norm)
        return result

    def setStrech(self):
        strechMode = self.getStrechMode()
        colorMode = self.getColorMode()
        zoomMode = self.getZoomMode()
        worker = Worker(self.calculateImage, self.image, strechMode, colorMode, zoomMode)
        worker.signals.result.connect(self.signalDisplayImage)
        self.threadpool.start(worker)

    def setColor(self):
        strechMode = self.getStrechMode()
        colorMode = self.getColorMode()
        zoomMode = self.getZoomMode()
        worker = Worker(self.calculateImage, self.image, strechMode, colorMode, zoomMode)
        worker.signals.result.connect(self.signalDisplayImage)
        self.threadpool.start(worker)

    def setZoom(self):
        strechMode = self.getStrechMode()
        colorMode = self.getColorMode()
        zoomMode = self.getZoomMode()
        worker = Worker(self.calculateImage, self.image, strechMode, colorMode, zoomMode)
        worker.signals.result.connect(self.signalDisplayImage)
        self.threadpool.start(worker)

    def getColorMode(self):
        if self.ui.btn_colorCool.isChecked():
            color = 'plasma'
        elif self.ui.btn_colorRainbow.isChecked():
            color = 'rainbow'
        elif self.ui.btn_colorSpectral.isChecked():
            color = 'nipy_spectral'
        else:
            color = 'gray'
        return color

    def getStrechMode(self):
        if self.ui.btn_strechLow.isChecked():
            strech = 'Low'
        elif self.ui.btn_strechMid.isChecked():
            strech = 'Mid'
        elif self.ui.btn_strechHigh.isChecked():
            strech = 'High'
        else:
            strech = 'Super'
        return strech

    def getZoomMode(self):
        if self.ui.btn_size12.isChecked():
            zoom = 12
        elif self.ui.btn_size25.isChecked():
            zoom = 25
        elif self.ui.btn_size50.isChecked():
            zoom = 50
        else:
            zoom = 100
        return zoom

    def setManualEnable(self, stat):
        self.ui.btn_expose.setEnabled(stat)
        self.ui.btn_solve.setEnabled(stat)
        self.ui.btn_cancel.setEnabled(stat)
        self.ui.btn_exposeCont.setEnabled(stat)
        self.ui.btn_loadFits.setEnabled(stat)
        if stat:
            self.setWindowTitle('Image Window')
        else:
            self.setWindowTitle('Image Window - Model Build Running - No manual exposures possible')

    def setCrosshairOnOff(self):
        if self.ui.checkShowCrosshairs.isChecked():
            self.ui.imageMarker.setVisible(True)
        else:
            self.ui.imageMarker.setVisible(False)

    @PyQt5.QtCore.pyqtSlot(str)
    def solveFitsImage(self, filename):
        self.imagePath = filename
        self.solveOnce()

    def exposeOnce(self):
        self.cancel = False
        # link to cam and check if available
        while not self.cancel:
            self.app.signalChangeStylesheet.emit(self.ui.btn_expose, 'running', True)
            if 'CONNECTION' in self.app.workerImaging.data:
                if self.app.workerImaging.data['CONNECTION']['CONNECT'] == 'Off':
                    break
            else:
                break
            # start prep imaging
            imageParams = dict()
            imageParams['Imagepath'] = ''
            imageParams['Exposure'] = self.app.ui.cameraExposure.value()
            imageParams['Directory'] = time.strftime('%Y-%m-%d', time.gmtime())
            imageParams['File'] = self.BASENAME + time.strftime('%H-%M-%S', time.gmtime()) + '.fit'
            self.app.messageQueue.put('#BWExposing Image: {0} for {1} seconds\n'.format(imageParams['File'], imageParams['Exposure']))
            self.imageReady = False
            self.app.workerImaging.imagingCommandQueue.put(imageParams)
            while not self.imageReady and not self.cancel:
                time.sleep(0.1)
                PyQt5.QtWidgets.QApplication.processEvents()
            if not os.path.isfile(imageParams['Imagepath']):
                self.app.messageQueue.put('#BWImaging failed\n')
                break
            self.signalShowFitsImage.emit(imageParams['Imagepath'])
            break
        self.app.signalChangeStylesheet.emit(self.ui.btn_expose, 'running', False)

    def solveOnce(self):
        self.cancel = False
        while not self.cancel:
            if self.imagePath == '':
                break
            if not os.path.isfile(self.imagePath):
                break
            self.app.signalChangeStylesheet.emit(self.ui.btn_solve, 'running', True)
            self.signalSetRaSolved.emit('')
            self.signalSetDecSolved.emit('')
            self.signalSetAngleSolved.emit('')
            imageParams = dict()
            imageParams['Imagepath'] = self.imagePath
            self.app.messageQueue.put('#BWSolving Image: {0}\n'.format(imageParams['Imagepath']))
            self.solveReady = False
            self.app.workerAstrometry.astrometryCommandQueue.put(imageParams)
            while not self.solveReady and not self.cancel:
                time.sleep(0.1)
                PyQt5.QtWidgets.QApplication.processEvents()
            if 'Solved' in imageParams:
                if imageParams['Solved']:
                    self.app.messageQueue.put('#BWSolving result: RA: {0}, DEC: {1}\n'.format(self.transform.decimalToDegree(imageParams['RaJ2000Solved'], False, False),
                                                                                              self.transform.decimalToDegree(imageParams['DecJ2000Solved'], True, False)))
                else:
                    self.app.messageQueue.put('#BWImage could not be solved: {0}\n'.format(imageParams['Message']))
            else:
                self.app.messageQueue.put('#BWSolve error\n')
            break
        self.app.signalChangeStylesheet.emit(self.ui.btn_solve, 'running', False)

    def exposeCont(self):
        self.cancel = False
        # link to cam and check if available
        while not self.cancel:
            self.app.signalChangeStylesheet.emit(self.ui.btn_exposeCont, 'running', True)
            if 'CONNECTION' in self.app.workerImaging.data:
                if self.app.workerImaging.data['CONNECTION']['CONNECT'] == 'Off':
                    break
            else:
                break
            # start prep imaging
            imageParams = dict()
            imageParams['Imagepath'] = ''
            imageParams['Exposure'] = self.app.ui.cameraExposure.value()
            imageParams['Directory'] = time.strftime('%Y-%m-%d', time.gmtime())
            imageParams['File'] = self.BASENAME + time.strftime('%H-%M-%S', time.gmtime()) + '.fit'
            self.app.messageQueue.put('#BWExposing Image: {0} for {1} seconds\n'.format(imageParams['File'], imageParams['Exposure']))
            self.imageReady = False
            self.app.workerImaging.imagingCommandQueue.put(imageParams)
            while not self.imageReady and not self.cancel:
                time.sleep(0.1)
                PyQt5.QtWidgets.QApplication.processEvents()
            if not os.path.isfile(imageParams['Imagepath']):
                self.app.messageQueue.put('#BWImaging failed\n')
                break
            self.signalShowFitsImage.emit(imageParams['Imagepath'])
        self.app.signalChangeStylesheet.emit(self.ui.btn_exposeCont, 'running', False)



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import copy
import shutil
import datetime
import time
import math
import PyQt5
import indi.indi_xml as indiXML
from analyse import analysedata
from modeling import model_points
from queue import Queue
from astrometry import transform
import astropy.io.fits as pyfits


class Slewpoint(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    queuePoint = Queue()
    signalStartSlewing = PyQt5.QtCore.pyqtSignal()
    signalPointImaged = PyQt5.QtCore.pyqtSignal(float, float)

    CYCLE = 200
    signalDestruct = PyQt5.QtCore.pyqtSignal()

    def __init__(self, main, thread):
        super().__init__()
        self.main = main
        self.thread = thread
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        self.mutexTakeNextPoint = PyQt5.QtCore.QMutex()
        self.isRunning = True
        self.takeNextPoint = False
        self.cycleTimer = None
        self.signalStartSlewing.connect(self.startSlewing)

    def startSlewing(self):
        self.mutexTakeNextPoint.lock()
        self.takeNextPoint = True
        self.mutexTakeNextPoint.unlock()

    def run(self):
        self.logger.info('model build slewpoint started')
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        self.queuePoint.queue.clear()

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        self.cycleTimer.stop()
        self.signalDestruct.disconnect(self.destruct)

    def doCommand(self):
        if self.takeNextPoint and not self.queuePoint.empty():
            self.mutexTakeNextPoint.lock()
            self.takeNextPoint = False
            self.mutexTakeNextPoint.unlock()
            modelingData = self.queuePoint.get()
            self.main.app.messageQueue.put('#BGSlewing to point {0:2d}  @ Az: {1:3.0f}\xb0 Alt: {2:2.0f}\xb0\n'.format(modelingData['Index'] + 1, modelingData['Azimuth'], modelingData['Altitude']))
            self.logger.info('Slewing to point {0:2d}  @ Az: {1:3.0f}\xb0 Alt: {2:2.0f}\xb0'.format(modelingData['Index'] + 1, modelingData['Azimuth'], modelingData['Altitude']))
            self.main.slewMountDome(modelingData)
            self.main.app.messageQueue.put('\tWait mount settling / delay time:  {0:02d} sec\n'.format(modelingData['SettlingTime']))
            self.main.app.messageQueue.put('Slewed>{0:02d}'.format(modelingData['Index'] + 1))
            time.sleep(modelingData['SettlingTime'])
            self.main.workerImage.queueImage.put(modelingData)
            # make signal for hemisphere that point is imaged
            self.signalPointImaged.emit(modelingData['Azimuth'], modelingData['Altitude'])


class Image(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    queueImage = Queue()
    signalImaging = PyQt5.QtCore.pyqtSignal()

    CYCLE = 200
    signalDestruct = PyQt5.QtCore.pyqtSignal()

    def __init__(self, main, thread):
        super().__init__()
        self.main = main
        self.thread = thread
        self.isRunning = True
        self.imageIntegrated = False
        self.imageSaved = False
        self.cycleTimer = None
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        self.mutexImageIntegrated = PyQt5.QtCore.QMutex()
        self.mutexImageSaved = PyQt5.QtCore.QMutex()
        self.main.app.workerImaging.imageIntegrated.connect(self.setImageIntegrated)
        self.main.app.workerImaging.imageSaved.connect(self.setImageSaved)

    def setImageIntegrated(self):
        self.mutexImageIntegrated.lock()
        self.imageIntegrated = True
        self.mutexImageIntegrated.unlock()

    def setImageSaved(self):
        self.mutexImageSaved.lock()
        self.imageSaved = True
        self.mutexImageSaved.unlock()

    def run(self):
        self.logger.info('model build imaging started')
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        # self.main.app.workerImaging.cameraHandler.cancel = True
        self.queueImage.queue.clear()

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        self.cycleTimer.stop()
        self.signalDestruct.disconnect(self.destruct)

    def doCommand(self):
        if not self.queueImage.empty():
            modelingData = self.queueImage.get()
            self.mutexImageSaved.lock()
            self.imageSaved = False
            self.mutexImageSaved.unlock()
            self.mutexImageIntegrated.lock()
            self.imageIntegrated = False
            self.mutexImageIntegrated.unlock()
            modelingData['File'] = 'Model_Image_' + '{0:03d}'.format(modelingData['Index']) + '.fit'
            modelingData['Imagepath'] = ''
            self.main.app.messageQueue.put('\tCapturing image for model point {0:2d}\n'.format(modelingData['Index'] + 1))
            self.logger.info('Capturing image for model point {0:2d}'.format(modelingData['Index'] + 1))
            # getting next image
            self.main.app.workerImaging.imagingCommandQueue.put(modelingData)
            # wait for imaging ready
            while not self.imageIntegrated and not self.main.cancel:
                time.sleep(0.1)
                PyQt5.QtWidgets.QApplication.processEvents()
            # next point after integrating but during downloading if possible or after IDLE
            self.main.workerSlewpoint.signalStartSlewing.emit()
            # we have to wait until image is downloaded before being able to plate solve
            while not self.imageSaved and not self.main.cancel:
                time.sleep(0.1)
                PyQt5.QtWidgets.QApplication.processEvents()
            self.main.app.messageQueue.put('Imaged>{0:02d}'.format(modelingData['Index'] + 1))
            self.logger.info('Imaged {0:02d}'.format(modelingData['Index'] + 1))
            self.main.workerPlatesolve.queuePlatesolve.put(copy.copy(modelingData))


class Platesolve(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    queuePlatesolve = Queue()

    CYCLE = 200
    signalDestruct = PyQt5.QtCore.pyqtSignal()

    def __init__(self, main, thread):
        super().__init__()
        self.main = main
        self.thread = thread
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        self.mutexImageDataDownloaded = PyQt5.QtCore.QMutex()
        self.isRunning = True
        self.cycleTimer = None
        self.imageDataDownloaded = False
        self.main.app.workerAstrometry.imageDataDownloaded.connect(self.setImageDataDownloaded)

    def setImageDataDownloaded(self):
        self.mutexImageDataDownloaded.lock()
        self.imageDataDownloaded = True
        self.mutexImageDataDownloaded.unlock()

    def run(self):
        self.logger.info('model build solving started')
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        self.queuePlatesolve.queue.clear()

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        self.cycleTimer.stop()
        self.signalDestruct.disconnect(self.destruct)

    def doCommand(self):
        if not self.queuePlatesolve.empty():
            self.mutexImageDataDownloaded.lock()
            self.imageDataDownloaded = False
            self.mutexImageDataDownloaded.unlock()
            modelingData = self.queuePlatesolve.get()
            if modelingData['Imagepath'] != '':
                self.main.app.messageQueue.put('\tSolving image for model point {0}\n'.format(modelingData['Index'] + 1))
                self.logger.info('Solving image for model point {0}'.format(modelingData['Index'] + 1))
                self.main.app.workerAstrometry.astrometryCommandQueue.put(modelingData)
                # wait for solving ready
                while not self.imageDataDownloaded and not self.main.cancel:
                    time.sleep(0.1)
                    PyQt5.QtWidgets.QApplication.processEvents()
                if 'RaJ2000Solved' in modelingData:
                    ra_sol_Jnow, dec_sol_Jnow = self.main.transform.transformERFA(modelingData['RaJ2000Solved'], modelingData['DecJ2000Solved'], 3)
                    modelingData['RaJNowSolved'] = ra_sol_Jnow
                    modelingData['DecJNowSolved'] = dec_sol_Jnow
                    modelingData['RaError'] = (modelingData['RaJ2000Solved'] - modelingData['RaJ2000']) * 3600
                    modelingData['DecError'] = (modelingData['DecJ2000Solved'] - modelingData['DecJ2000']) * 3600
                    modelingData['ModelError'] = math.sqrt(modelingData['RaError'] * modelingData['RaError'] + modelingData['DecError'] * modelingData['DecError'])
                    modelingData['Message'] = 'OK - solved'
                    self.main.app.messageQueue.put('\tImage path: {0}\n'.format(modelingData['Imagepath']))
                    self.main.app.messageQueue.put('\tRA_diff:  {0:2.1f}    DEC_diff: {1:2.1f}\n'.format(modelingData['RaError'], modelingData['DecError']))
                    self.logger.info('RA_diff:  {0:2.1f}    DEC_diff: {1:2.1f}, image path: {2}'.format(modelingData['RaError'], modelingData['DecError'], modelingData['Imagepath']))
                    self.main.solvedPointsQueue.put(copy.copy(modelingData))
                else:
                    if 'Message' in modelingData:
                        self.main.app.messageQueue.put('\tSolving error for point {0}: {1}\n'.format(modelingData['Index'] + 1, modelingData['Message']))
                        self.logger.warning('Solving error for point {0}: {1}'.format(modelingData['Index'] + 1, modelingData['Message']))
                    else:
                        self.main.app.messageQueue.put('\tSolving canceled\n')
                        self.logger.warning('Solving canceled')
            # write progress to hemisphere windows
            self.main.app.messageQueue.put('Solved>{0:02d}'.format(modelingData['Index'] + 1))
            # write progress estimation to main gui
            modelingDone = (modelingData['Index'] + 1) / modelingData['NumberPoints']
            timeElapsed = time.time() - self.main.timeStart
            if modelingDone != 0:
                timeEstimation = (1 / modelingDone * timeElapsed) * (1 - modelingDone)
            else:
                timeEstimation = 0
            self.main.app.messageQueue.put('percent{0:4.3f}'.format(modelingDone))
            self.main.app.messageQueue.put('timeEst{0}'.format(time.strftime('%M:%S', time.gmtime(timeEstimation))))
            finished = datetime.timedelta(seconds=timeEstimation) + datetime.datetime.now()
            self.main.app.messageQueue.put('timeFin{0}'.format(finished.strftime('%H:%M:%S')))
            # we come to an end
            if modelingData['NumberPoints'] == modelingData['Index'] + 1:
                self.main.modelingHasFinished = True


class ModelingBuild:
    logger = logging.getLogger(__name__)

    def __init__(self, app):
        # make environment available to class
        self.app = app

        # assign support classes
        self.analyseData = analysedata.Analyse(self.app)
        self.transform = transform.Transform(self.app)
        self.modelPoints = model_points.ModelPoints(self.app)

        # initialize the parallel thread modeling parts
        self.threadSlewpoint = PyQt5.QtCore.QThread()
        self.workerSlewpoint = Slewpoint(self, self.threadSlewpoint)
        self.workerSlewpoint.moveToThread(self.threadSlewpoint)
        self.threadSlewpoint.started.connect(self.workerSlewpoint.run)

        self.threadImage = PyQt5.QtCore.QThread()
        self.workerImage = Image(self, self.threadImage)
        self.workerImage.moveToThread(self.threadImage)
        self.threadImage.started.connect(self.workerImage.run)

        self.threadPlatesolve = PyQt5.QtCore.QThread()
        self.workerPlatesolve = Platesolve(self, self.threadPlatesolve)
        self.workerPlatesolve.moveToThread(self.threadPlatesolve)
        self.threadPlatesolve.started.connect(self.workerPlatesolve.run)

        # class variables
        self.solvedPointsQueue = Queue()
        self.timeStart = 0
        self.results = []
        self.modelingResultData = []
        self.modelAlignmentData = []
        self.modelRun = False
        self.modelingHasFinished = False
        self.numberPointsMax = 0
        self.numberSolvedPoints = 0
        self.cancel = False
        self.imageReady = False
        self.solveReady = False
        self.mountSlewFinished = False
        self.domeSlewFinished = False

        # signal slot
        self.app.workerMountDispatcher.signalSlewFinished.connect(self.setMountSlewFinished)
        self.app.workerDome.signalSlewFinished.connect(self.setDomeSlewFinished)
        self.app.workerImaging.imageSaved.connect(self.setImageReady)
        self.app.workerAstrometry.imageDataDownloaded.connect(self.setSolveReady)

    def initConfig(self):
        self.modelPoints.initConfig()

    def storeConfig(self):
        self.modelPoints.storeConfig()

    def setCancel(self):
        self.cancel = True

    def setImageReady(self):
        self.imageReady = True

    def setSolveReady(self):
        self.solveReady = True

    def setMountSlewFinished(self):
        self.mountSlewFinished = True

    def setDomeSlewFinished(self):
        self.domeSlewFinished = True

    def clearAlignmentModel(self):
        # clearing the older results, because they are invalid afterwards
        self.modelingResultData = []
        # clearing the mount model and wait 4 seconds for the mount computer to recover (I don't know why, but Per Frejval did it)
        self.app.mountCommandQueue.put(':delalig#')
        time.sleep(4)

    def slewMountDome(self, modelingData):
        altitude = modelingData['Altitude']
        azimuth = modelingData['Azimuth']
        self.mountSlewFinished = False
        self.domeSlewFinished = False
        # limit azimuth and altitude
        if azimuth >= 360:
            azimuth = 359.9
        elif azimuth < 0.0:
            azimuth = 0.0
        # setting the coordinates for the mount
        self.app.mountCommandQueue.put(':Sz{0:03d}*{1:02d}#'.format(int(azimuth), int((azimuth - int(azimuth)) * 60 + 0.5)))
        self.app.mountCommandQueue.put(':Sa+{0:02d}*{1:02d}#'.format(int(altitude), int((altitude - int(altitude)) * 60 + 0.5)))
        self.app.mountCommandQueue.put(':MS#')
        # if there is a dome connected, we have to start slewing it, too
        if modelingData['DomeIsConnected']:
            self.app.domeCommandQueue.put(('SlewAzimuth', azimuth))
            while not self.domeSlewFinished and not self.mountSlewFinished:
                if self.cancel:
                    self.logger.info('Modeling cancelled in loop mount and dome wait while for stop slewing')
                    break
                time.sleep(0.2)
        else:
            while not self.mountSlewFinished:
                if self.cancel:
                    self.logger.info('Modeling cancelled in loop mount wait while for stop slewing')
                    break
                time.sleep(0.2)

    def runModelCore(self, messageQueue, runPoints, modelingData):
        self.app.imageWindow.signalSetManualEnable.emit(False)
        # start clearing the data
        results = []
        # preparing the gui outputs
        messageQueue.put('Imaged>{0:02d}'.format(0))
        messageQueue.put('Solved>{0:02d}'.format(0))
        messageQueue.put('Slewed>{0:02d}'.format(0))
        messageQueue.put('percent0')
        messageQueue.put('timeEla--:--')
        messageQueue.put('timeEst--:--')
        messageQueue.put('timeFin--:--:--')
        self.logger.info('modelingData: {0}'.format(modelingData))
        # start tracking
        self.app.mountCommandQueue.put(':PO#')
        self.app.mountCommandQueue.put(':AP#')
        self.workerSlewpoint.mutexTakeNextPoint.lock()
        self.workerSlewpoint.takeNextPoint = False
        self.workerSlewpoint.mutexTakeNextPoint.unlock()
        self.modelRun = True
        self.timeStart = time.time()
        # starting the necessary threads
        self.threadSlewpoint.start()
        self.threadImage.start()
        self.threadPlatesolve.start()
        # wait until threads started
        while not self.workerImage.isRunning and not self.workerPlatesolve.isRunning and not self.workerSlewpoint.isRunning:
            time.sleep(0.2)
        if len(runPoints) > 100:
            messageQueue.put('#BYMore than 100 points defined, using only first 100 points for model build\n')
            messageQueue.put('ToModel>{0:02d}'.format(100))
        # loading the points to the queue, but only the first 100, because mount computer does only allow 100 points
        for i, (p_az, p_alt) in enumerate(runPoints[:100]):
            modelingData['Index'] = i
            modelingData['Azimuth'] = p_az
            modelingData['Altitude'] = p_alt
            modelingData['NumberPoints'] = len(runPoints)
            # has to be a copy, otherwise we have always the same content
            self.workerSlewpoint.queuePoint.put(copy.copy(modelingData))
        # start process
        self.modelingHasFinished = False
        self.timeStart = time.time()
        self.workerSlewpoint.signalStartSlewing.emit()
        while self.modelRun:
            # stop loop if modeling is cancelled from external
            if self.cancel:
                self.app.workerAstrometry.astrometryCancel.emit()
                self.app.workerImaging.imagingCancel.emit()
                break
            # stop loop if finished
            if self.modelingHasFinished:
                break
            timeElapsed = time.time() - self.timeStart
            messageQueue.put('timeEla{0}'.format(time.strftime('%M:%S', time.gmtime(timeElapsed))))
            time.sleep(0.2)
            PyQt5.QtWidgets.QApplication.processEvents()
        if self.cancel:
            # clearing the gui
            messageQueue.put('percent0')
            messageQueue.put('timeEst--:--')
            self.logger.info('Modeling cancelled in main loop')
        self.workerSlewpoint.stop()
        self.workerImage.stop()
        self.workerPlatesolve.stop()
        self.modelRun = False
        while not self.solvedPointsQueue.empty():
            modelingData = self.solvedPointsQueue.get()
            # clean up intermediate data
            results.append(copy.copy(modelingData))
        if 'KeepImages' and 'BaseDirImages' in modelingData:
            if not modelingData['KeepImages']:
                shutil.rmtree(modelingData['BaseDirImages'], ignore_errors=True)
        # turn list of dicts to dict of lists
        if len(results) > 0:
            changedResults = dict(zip(results[0], zip(*[d.values() for d in results])))
        else:
            changedResults = {}
        self.app.imageWindow.signalSetManualEnable.emit(True)
        return changedResults

    def runInitialModel(self):
        modelingData = {'Directory': time.strftime("%Y-%m-%d-%H-%M-%S", time.gmtime())}
        # imaging has to be connected
        if 'CONNECTION' not in self.app.workerImaging.cameraHandler.data:
            return
        if self.app.workerImaging.data['CONNECTION']['CONNECT'] == 'Off':
            return
        # solver has to be connected
        if 'CONNECTION' not in self.app.workerAstrometry.astrometryHandler.data:
            return
        if self.app.workerAstrometry.data['CONNECTION']['CONNECT'] == 'Off':
            return
        # telescope has to be connected
        if not self.app.workerMountDispatcher.mountStatus['Command']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Once']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Slow']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Medium']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Fast']:
            return
        if not self.app.workerMountDispatcher.mountStatus['GetAlign']:
            return
        if not self.app.workerMountDispatcher.mountStatus['SetAlign']:
            return
        # there have to be some modeling points
        if len(self.modelPoints.modelPoints) == 0:
            self.logger.warning('There are no modeling points to process')
            return
        # if dome is present, it has to be connected, too
        if not self.app.ui.pd_chooseDome.currentText().startswith('No Dome'):
            domeIsConnected = self.app.workerDome.data['Connected']
        else:
            domeIsConnected = False
        modelingData['DomeIsConnected'] = domeIsConnected
        modelingData['SettlingTime'] = int(self.app.ui.settlingTime.value())
        modelingData['KeepImages'] = self.app.ui.checkKeepImages.isChecked()
        self.app.workerImaging.cameraHandler.cancel = False
        self.app.workerAstrometry.astrometryHandler.cancel = False
        self.cancel = False
        # now starting work
        timeStartModeling = time.time()
        self.app.messageQueue.put('#BWStart Initial Model\n')
        self.app.workerMountDispatcher.mountModelHandling.saveModel('BACKUP')
        self.modelAlignmentData = self.runModelCore(self.app.messageQueue, self.modelPoints.modelPoints, modelingData)
        self.app.messageQueue.put('#BWModel processed\n')
        name = modelingData['Directory'] + '_initial'
        if len(self.modelAlignmentData) > 0:
            self.app.messageQueue.put('Programming model to mount\n')
            self.app.workerMountDispatcher.programBatchData(self.modelAlignmentData)
            self.app.messageQueue.put('Reloading actual alignment model from mount\n')
            self.app.workerMountDispatcher.reloadAlignmentModel()
            self.app.messageQueue.put('Syncing actual alignment model and modeling data\n')
            if self.app.workerMountDispatcher.retrofitMountData(self.modelAlignmentData):
                self.analyseData.saveData(self.modelAlignmentData, name)
                self.app.signalSetAnalyseFilename.emit(name)
                if self.app.analyseWindow.showStatus:
                    self.app.ui.btn_openAnalyseWindow.clicked.emit()
                self.app.audioCommandQueue.put('ModelingFinished')
                self.app.workerMountDispatcher.mountModelHandling.saveModel('INITIAL')
                self.app.messageQueue.put('#BGInitial Model finished with success, runtime: {0} (MM:SS)\n'.format(time.strftime("%M:%S", time.gmtime(time.time() - timeStartModeling))))
                self.logger.info('Initial Model finished with success, runtime: {0} (MM:SS)'.format(time.strftime("%M:%S", time.gmtime(time.time() - timeStartModeling))))
            else:
                self.app.messageQueue.put('#BRModel finished with errors\n')
                self.logger.warning('Model finished with errors')
        else:
            self.app.messageQueue.put('#BRModel finished with errors\n')
            self.logger.warning('Model finished with errors')

    def runFullModel(self):
        modelingData = {'Directory': time.strftime("%Y-%m-%d-%H-%M-%S", time.gmtime())}
        # imaging has to be connected
        if 'CONNECTION' not in self.app.workerImaging.data:
            return
        if self.app.workerImaging.data['CONNECTION']['CONNECT'] == 'Off':
            return
        # solver has to be connected
        if 'CONNECTION' not in self.app.workerAstrometry.data:
            return
        if self.app.workerAstrometry.data['CONNECTION']['CONNECT'] == 'Off':
            return
        # telescope has to be connected
        if not self.app.workerMountDispatcher.mountStatus['Command']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Once']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Slow']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Medium']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Fast']:
            return
        if not self.app.workerMountDispatcher.mountStatus['GetAlign']:
            return
        if not self.app.workerMountDispatcher.mountStatus['SetAlign']:
            return
        # there have to be some modeling points
        if len(self.modelPoints.modelPoints) == 0:
            self.logger.warning('There are no modeling points to process')
            return
        # if dome is present, it has to be connected, too
        if not self.app.ui.pd_chooseDome.currentText().startswith('No Dome'):
            domeIsConnected = self.app.workerDome.data['Connected']
        else:
            domeIsConnected = False
        modelingData['DomeIsConnected'] = domeIsConnected
        modelingData['SettlingTime'] = int(self.app.ui.settlingTime.value())
        modelingData['KeepImages'] = self.app.ui.checkKeepImages.isChecked()
        self.app.workerImaging.cameraHandler.cancel = False
        self.app.workerAstrometry.astrometryHandler.cancel = False
        self.cancel = False
        # now starting work
        timeStartModeling = time.time()
        self.app.messageQueue.put('#BWStart Full Model\n')
        self.app.workerMountDispatcher.mountModelHandling.saveModel('BACKUP')
        self.modelAlignmentData = self.runModelCore(self.app.messageQueue, self.modelPoints.modelPoints, modelingData)
        self.app.messageQueue.put('#BWModel processed\n')
        name = modelingData['Directory'] + '_full'
        if len(self.modelAlignmentData) > 0:
            self.app.messageQueue.put('Programming model to mount\n')
            self.app.workerMountDispatcher.programBatchData(self.modelAlignmentData)
            self.app.messageQueue.put('Reloading actual alignment model from mount\n')
            self.app.workerMountDispatcher.reloadAlignmentModel()
            self.app.messageQueue.put('Syncing actual alignment model and modeling data\n')
            if self.app.workerMountDispatcher.retrofitMountData(self.modelAlignmentData):
                self.analyseData.saveData(self.modelAlignmentData, name)
                self.app.signalSetAnalyseFilename.emit(name)
                if self.app.analyseWindow.showStatus:
                    self.app.ui.btn_openAnalyseWindow.clicked.emit()
                self.app.audioCommandQueue.put('ModelingFinished')
                self.app.workerMountDispatcher.mountModelHandling.saveModel('FULL')
                self.app.messageQueue.put('#BGFull Model finished with success, runtime: {0} (MM:SS)\n'.format(time.strftime('%M:%S', time.gmtime(time.time() - timeStartModeling))))
                self.logger.info('Full Model finished with success, runtime: {0} (MM:SS)'.format(time.strftime('%M:%S', time.gmtime(time.time() - timeStartModeling))))
            else:
                self.app.messageQueue.put('#BRModel finished with errors\n')
                self.logger.warning('Model finished with errors')
        else:
            self.app.messageQueue.put('#BRModel finished with errors\n')
            self.logger.warning('Model finished with errors')

    def plateSolveSync(self):
        self.app.messageQueue.put('#BWStart Sync Mount Model\n')
        # link to cam and check if available
        if 'CONNECTION' in self.app.workerImaging.data:
            if self.app.workerImaging.data['CONNECTION']['CONNECT'] == 'Off':
                return
        else:
            return
        # start prep imaging
        self.app.mountCommandQueue.put(':PO#')
        self.app.mountCommandQueue.put(':AP#')
        imageParams = dict()
        imageParams['Imagepath'] = ''
        imageParams['Exposure'] = self.app.ui.cameraExposure.value()
        imageParams['Directory'] = time.strftime('%Y-%m-%d', time.gmtime())
        imageParams['File'] = 'platesolvesync.fit'
        self.app.messageQueue.put('#BWExposing Image: {0} for {1} seconds\n'.format(imageParams['File'], imageParams['Exposure']))
        self.imageReady = False
        self.app.workerImaging.imagingCommandQueue.put(imageParams)
        while not self.imageReady and not self.cancel:
            time.sleep(0.1)
            PyQt5.QtWidgets.QApplication.processEvents()
        self.app.messageQueue.put('#BWSolving Image: {0}\n'.format(imageParams['Imagepath']))
        # wait for solving
        self.solveReady = False
        self.app.workerAstrometry.astrometryCommandQueue.put(imageParams)
        while not self.solveReady and not self.cancel:
            time.sleep(0.1)
            PyQt5.QtWidgets.QApplication.processEvents()
        if 'Solved' in imageParams:
            if imageParams['Solved']:
                self.app.messageQueue.put('#BWSolving result: RA: {0}, DEC: {1}\n'.format(self.transform.decimalToDegree(imageParams['RaJ2000Solved'], False, False),
                                                                                          self.transform.decimalToDegree(imageParams['DecJ2000Solved'], True, False)))
                ra_sol_Jnow, dec_sol_Jnow = self.transform.transformERFA(imageParams['RaJ2000Solved'], imageParams['DecJ2000Solved'], 3)
                ra_form = self.transform.decimalToDegree(ra_sol_Jnow, False, False)
                dec_form = self.transform.decimalToDegree(dec_sol_Jnow, True, False)
                success = self.app.workerMountDispatcher.syncMountModel(ra_form, dec_form)
                if success:
                    self.app.messageQueue.put('\tMount Model Synced\n')
                else:
                    self.app.messageQueue.put('\tMount Model could not be synced - please check!\n')
            else:
                self.logger.warning('Solve key in imageParams missing')
        else:
            self.app.messageQueue.put('\tSolving error: {0}\n'.format(mes))
        if not self.app.ui.checkKeepImages.isChecked():
            shutil.rmtree(imageParams['BaseDirImages'], ignore_errors=True)
        self.app.messageQueue.put('#BWSync Mount Model finished !\n')

    def runFlexure(self):
        modelingData = {'Directory': time.strftime("%Y-%m-%d-%H-%M-%S", time.gmtime())}
        # imaging has to be connected
        if 'CONNECTION' not in self.app.workerImaging.cameraHandler.data:
            return
        if self.app.workerImaging.data['CONNECTION']['CONNECT'] == 'Off':
            return
        # solver has to be connected
        if 'CONNECTION' not in self.app.workerAstrometry.astrometryHandler.data:
            return
        if self.app.workerAstrometry.data['CONNECTION']['CONNECT'] == 'Off':
            return
        # telescope has to be connected
        if not self.app.workerMountDispatcher.mountStatus['Command']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Once']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Slow']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Medium']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Fast']:
            return
        if not self.app.workerMountDispatcher.mountStatus['GetAlign']:
            return
        if not self.app.workerMountDispatcher.mountStatus['SetAlign']:
            return
        # calculate model points
        self.modelPoints.modelPoints = []
        for i in range(0, int(self.app.ui.numberRunsTimeChange.value())):
            self.modelPoints.modelPoints.append((int(self.app.ui.azimuthTimeChange.value()), int(self.app.ui.altitudeTimeChange.value())))
        # if dome is present, it has to be connected, too
        if not self.app.ui.pd_chooseDome.currentText().startswith('No Dome'):
            domeIsConnected = self.app.workerDome.data['Connected']
        else:
            domeIsConnected = False
        modelingData['DomeIsConnected'] = domeIsConnected
        modelingData['SettlingTime'] = int(self.app.ui.delayTimeFlexure.value())
        modelingData['KeepImages'] = self.app.ui.checkKeepImages.isChecked()
        self.app.workerImaging.cameraHandler.cancel = False
        self.app.workerAstrometry.astrometryHandler.cancel = False
        self.cancel = False
        # now starting work
        timeStartModeling = time.time()
        self.app.messageQueue.put('#BWStart Flexure\n')
        self.modelAlignmentData = self.runModelCore(self.app.messageQueue, self.modelPoints.modelPoints, modelingData)
        self.app.messageQueue.put('#BWFlexure processed\n')
        name = modelingData['Directory'] + '_flexure'
        if len(self.modelAlignmentData) > 0:
            self.analyseData.saveData(self.modelAlignmentData, name)
            self.app.signalSetAnalyseFilename.emit(name)
            if self.app.analyseWindow.showStatus:
                self.app.ui.btn_openAnalyseWindow.clicked.emit()
            self.app.audioCommandQueue.put('ModelingFinished')
            self.app.messageQueue.put('#BGFlexure finished with success, runtime: {0} (MM:SS)\n'.format(time.strftime('%M:%S', time.gmtime(time.time() - timeStartModeling))))
            self.logger.info('Flexure finished with success, runtime: {0} (MM:SS)'.format(time.strftime('%M:%S', time.gmtime(time.time() - timeStartModeling))))
        else:
            self.app.messageQueue.put('#BRFlexure finished with errors\n')
            self.logger.warning('Flexure finished with errors')

    def runHysterese(self):
        modelingData = {'Directory': time.strftime("%Y-%m-%d-%H-%M-%S", time.gmtime())}
        # imaging has to be connected
        if 'CONNECTION' not in self.app.workerImaging.cameraHandler.data:
            return
        if self.app.workerImaging.data['CONNECTION']['CONNECT'] == 'Off':
            return
        # solver has to be connected
        if 'CONNECTION' not in self.app.workerAstrometry.astrometryHandler.data:
            return
        if self.app.workerAstrometry.data['CONNECTION']['CONNECT'] == 'Off':
            return
        # telescope has to be connected
        if not self.app.workerMountDispatcher.mountStatus['Command']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Once']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Slow']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Medium']:
            return
        if not self.app.workerMountDispatcher.mountStatus['Fast']:
            return
        if not self.app.workerMountDispatcher.mountStatus['GetAlign']:
            return
        if not self.app.workerMountDispatcher.mountStatus['SetAlign']:
            return
        # calculate model points
        alt1 = int(self.app.ui.altitudeHysterese1.value())
        alt2 = int(self.app.ui.altitudeHysterese2.value())
        az1 = int(self.app.ui.azimuthHysterese1.value())
        az2 = int(self.app.ui.azimuthHysterese2.value())
        numberRunsHysterese = int(self.app.ui.numberRunsHysterese.value())
        self.modelPoints.modelPoints = []
        for i in range(0, numberRunsHysterese):
            self.modelPoints.modelPoints.append((az1, alt1))
            self.modelPoints.modelPoints.append((az2, alt2))
        # if dome is present, it has to be connected, too
        if not self.app.ui.pd_chooseDome.currentText().startswith('No Dome'):
            domeIsConnected = self.app.workerDome.data['Connected']
        else:
            domeIsConnected = False
        modelingData['DomeIsConnected'] = domeIsConnected
        modelingData['SettlingTime'] = int(self.app.ui.delayTimeHysterese.value())
        modelingData['KeepImages'] = self.app.ui.checkKeepImages.isChecked()
        self.app.workerImaging.cameraHandler.cancel = False
        self.app.workerAstrometry.astrometryHandler.cancel = False
        self.cancel = False
        # now starting work
        timeStartModeling = time.time()
        self.app.messageQueue.put('#BWStart Flexure\n')
        self.modelAlignmentData = self.runModelCore(self.app.messageQueue, self.modelPoints.modelPoints, modelingData)
        self.app.messageQueue.put('#BWFlexure processed\n')
        name = modelingData['Directory'] + '_flexure'
        if len(self.modelAlignmentData) > 0:
            self.analyseData.saveData(self.modelAlignmentData, name)
            self.app.signalSetAnalyseFilename.emit(name)
            if self.app.analyseWindow.showStatus:
                self.app.ui.btn_openAnalyseWindow.clicked.emit()
            self.app.audioCommandQueue.put('ModelingFinished')
            self.app.messageQueue.put('#BGFlexure finished with success, runtime: {0} (MM:SS)\n'.format(time.strftime('%M:%S', time.gmtime(time.time() - timeStartModeling))))
            self.logger.info('Flexure finished with success, runtime: {0} (MM:SS)'.format(time.strftime('%M:%S', time.gmtime(time.time() - timeStartModeling))))
        else:
            self.app.messageQueue.put('#BRFlexure finished with errors\n')
            self.logger.warning('Flexure finished with errors')
